unit uno_any2;

interface

{$I uno_common.inc}

uses
   uno_sal_config
  ,uno_sal_types
  ,uno_data
  ,uno_typelib_typedescription
  ,uno_uno_mapping;

type
  P_uno_Any  = ^uno_Any;
  PP_uno_Any  = ^P_uno_Any;

  // This is the binary specification of an UNO any.
  _uno_Any = record
    // type of value
    pType: p_typelib_TypeDescriptionReference;
    // pointer to value; this may point to pReserved and thus the uno_Any is not anytime
    // mem-copyable! You may have to correct the pData pointer to pReserved. Otherwise you need
    // not, because the data is stored in heap space.
    pData: Pointer; 
    // reserved space for storing value
    pReserved: Pointer; 
  end;
  uno_Any = _uno_Any; 
  
  (** Assign an any with a given value. Interfaces are acquired or released by the given callback
    functions.
    
   @param pDest   pointer memory of destination any
   @param pSource   pointer to source value; defaults (0) to default constructed value
   @param pTypeDescr  type description of value; defaults (0) to void
   @param acquire   function called each time an interface needs to be acquired;
                defaults (0) to uno
   @param release   function called each time an interface needs to be released;
                defaults (0) to uno
  *)
  procedure uno_any_assign(pDest: p_uno_Any;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;
                            acquire: uno_AcquireFunc;  release: uno_ReleaseFunc); cdecl;
  
  (** Assign an any with a given value. Interfaces are acquired or released by the given callback
    functions.
    
   @param pDest   pointer memory of destination any
   @param pSource   pointer to source value; defaults (0) to default constructed value
   @param pTypeDescr  type description of value; defaults (0) to void
   @param acquire   function called each time an interface needs to be acquired;
                defaults (0) to uno
   @param release   function called each time an interface needs to be released;
                defaults (0) to uno
  *)
  procedure uno_type_any_assign(pDest: p_uno_Any;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;
                                 acquire: uno_AcquireFunc;  release: uno_ReleaseFunc); cdecl;
  
  (** Constructs an any with a given value. Interfaces are acquired by the given callback function.
    
   @param pDest   pointer memory of destination any
   @param pSource   pointer to source value; defaults (0) to default constructed value
   @param pTypeDescr  type description of value; defaults (0) to void
   @param acquire   function called each time an interface needs to be acquired;
                defaults (0) to uno
  *)
  procedure uno_any_construct(pDest: p_uno_Any;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;
                                 acquire: uno_AcquireFunc); cdecl; 

  (** Constructs an any with a given value. Interfaces are acquired by the given callback function.
   
   @param pDest   pointer memory of destination any
   @param pSource   pointer to source value; defaults (0) to default constructed value
   @param pType   type of value; defaults (0) to void
   @param acquire   function called each time an interface needs to be acquired;
                defaults (0) to uno
  *)

  procedure uno_type_any_construct(pDest: p_uno_Any;  pSource: Pointer; pType: p_typelib_TypeDescriptionReference;
                                    acquire: uno_AcquireFunc); cdecl; 
  
  (** Constructs an any with a given value and converts/ maps interfaces.
    
   @param pDest   pointer memory of destination any
   @param pSource   pointer to source value; defaults (0) to default constructed value
   @param pTypeDescr  type description of value; defaults (0) to void
   @param mapping   mapping to convert/ map interfaces
  *)
  procedure uno_any_constructAndConvert(pDest: p_uno_Any;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;
                                           mapping: p_uno_Mapping); cdecl;
  
  (** Constructs an any with a given value and converts/ maps interfaces.
    
   @param pDest   pointer memory of destination any
   @param pSource   pointer to source value; defaults (0) to default constructed value
   @param pType   type of value; defaults (0) to void
   @param mapping   mapping to convert/ map interfaces
  *)
  procedure uno_type_any_constructAndConvert(pDest: p_uno_Any;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;
                                                mapping: p_uno_Mapping); cdecl; 
  
  (** Destructs an any.
    
   @param pValue   pointer to any
   @param release   function called each time an interface needs to be released;
                defaults (0) to uno
  *)
  procedure uno_any_destruct(pValue: p_uno_Any;  release: uno_ReleaseFunc); cdecl;
  
  (** Sets value to void.
    
   @param pValue   pointer to any
   @param release   function called each time an interface needs to be released;
                defaults (0) to uno
  *)
  procedure uno_any_clear(pValue: p_uno_Any;  release: uno_ReleaseFunc); cdecl;
  
  
implementation
  
  procedure uno_any_assign(pDest: p_uno_Any;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;  acquire: uno_AcquireFunc;  release: uno_ReleaseFunc);  external cppu3LibraryName;
  procedure uno_type_any_assign(pDest: p_uno_Any;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;  acquire: uno_AcquireFunc;  release: uno_ReleaseFunc);  external cppu3LibraryName;
  procedure uno_any_construct(pDest: p_uno_Any;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;  acquire: uno_AcquireFunc);  external cppu3LibraryName;
  procedure uno_type_any_construct(pDest: p_uno_Any;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;  acquire: uno_AcquireFunc);  external cppu3LibraryName;
  procedure uno_any_constructAndConvert(pDest: p_uno_Any;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;  mapping: p_uno_Mapping);  external cppu3LibraryName;
  procedure uno_type_any_constructAndConvert(pDest: p_uno_Any;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;  mapping: p_uno_Mapping);  external cppu3LibraryName;
  procedure uno_any_destruct(pValue: p_uno_Any;  release: uno_ReleaseFunc); external cppu3LibraryName;
  procedure uno_any_clear(pValue: p_uno_Any;  release: uno_ReleaseFunc); external cppu3LibraryName; 

end.
