unit uno_data;

interface

{$I uno_common.inc}

uses
   uno_sal_config
  ,uno_typelib_typedescription
  ,uno_sal_types
  ,uno_uno_mapping;

type
  (** Generic function pointer declaration to query for an interface.
   @param pInterface interface
   @param pTypedemanded interface type
   @return interface pointer
  *)
    uno_QueryInterfaceFunc = function(pInterface: Pointer; pType: p_typelib_TypeDescriptionReference): Pointer; cdecl;

  (** Generic function pointer declaration to acquire an interface.
   @param pInterface interface to be acquired
  *)
    uno_AcquireFunc = procedure( pInterface: Pointer ); cdecl;

  (** Generic function pointer declaration to release an interface.
   @param pInterface interface to be release
  *)
    uno_ReleaseFunc = procedure( pInterface: Pointer ); cdecl;

  (** Tests if two values are equal. May compare different types (e.g., short to long).
   @param pVal1   pointer to a value
   @param pVal1TypeDescr type description of pVal1
   @param pVal2   pointer to another value
   @param pVal2TypeDescr type description of pVal2
   @param queryInterface function called each time two interfaces are tested whether they belong
       to the same object; defaults (0) to uno
   @param release   function to release queried interfaces; defaults (0) to uno
   @return true if values are equal
  *)
  function uno_equalData(pVal1: Pointer;  pVal1TypeDescr: p_typelib_TypeDescription;
              pVal2: Pointer;  pVal2TypeDescr: p_typelib_TypeDescription;
              queryInterface: uno_QueryInterfaceFunc;  release: uno_ReleaseFunc): sal_Bool; cdecl;

  (** Tests if two values are equal. May compare different types (e.g., short to long).
   @param pVal1   pointer to a value
   @param pVal1Type  type of pVal1
   @param pVal2   pointer to another value
   @param pVal2Type  type of pVal2
   @param queryInterface function called each time two interfaces are tested whether they belong
       to the same object; defaults (0) to uno
   @param release   function to release queried interfaces; defaults (0) to uno
   @return true if values are equal
  *)  
  function uno_type_equalData(pVal1: Pointer;  pVal1Type: p_typelib_TypeDescriptionReference;  
                 pVal2: Pointer;  pVal2Type: p_typelib_TypeDescriptionReference;
                 queryInterface: uno_QueryInterfaceFunc;  release: uno_ReleaseFunc): sal_Bool; cdecl;
  
  (** Copy construct memory with given value. The size of the destination value must be larger
    or equal to the size of the source value.
   @param pDest   pointer to destination value memory
   @param pSource   pointer to source value
   @param pTypeDescr  type description of source
   @param acquire   function called each time an interface needs to be acquired;
                defaults (0) to uno
  *)  
  procedure uno_copyData(pDest: Pointer;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;
                            acquire: uno_AcquireFunc); cdecl;

  (** Copy construct memory with given value. The size of the destination value must be larger
    or equal to the size of the source value.
    
   @param pDest   pointer to destination value memory
   @param pSource   pointer to source value
   @param pType   type of source
   @param acquire   function called each time an interface needs to be acquired;
                defaults (0) to uno
  *)  
  procedure uno_type_copyData(pDest: Pointer;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;
                                 acquire: uno_AcquireFunc); cdecl; 
  
  (** Copy construct memory with given value. The size of the destination value must be larger
    or equal to the size of the source value. Interfaces are converted/ mapped by mapping parameter.
    
   @param pDest   pointer to destination value memory
   @param pSource   pointer to source value
   @param pTypeDescr  type description of source
   @param mapping   mapping to convert/ map interfaces
  *)  

  procedure uno_copyAndConvertData(pDest: Pointer;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;
                                      mapping: p_uno_Mapping); cdecl; 
  (** Copy construct memory with given value. The size of the destination value must be larger
    or equal to the size of the source value. Interfaces are converted/ mapped by mapping parameter.
    
   @param pDest   pointer to destination value memory
   @param pSource   pointer to source value
   @param pType   type of source
   @param mapping   mapping to convert/ map interfaces
  *)  
  procedure uno_type_copyAndConvertData(pDest: Pointer;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;
                                           mapping: p_uno_Mapping); cdecl;
  
  (** Destructs a given value; does NOT free its memory!
    
   @param pValue   value to be destructed
   @param pTypeDescr  type description of value
   @param release   function called each time an interface pointer needs to be released;
                defaults (0) to uno
  *)  
  procedure uno_destructData(pValue: Pointer; pTypeDescr: p_typelib_TypeDescription; release: uno_ReleaseFunc); cdecl;

  (** Destructs a given value; does NOT free its memory!
    
   @param pValue   value to be destructed
   @param pType   type of value
   @param release   function called each time an interface pointer needs to be released;
                defaults (0) to uno
  *)  
  procedure uno_type_destructData(pValue: Pointer;  pType: p_typelib_TypeDescriptionReference; release: uno_ReleaseFunc); cdecl; 
  
  (** Default constructs a value. All simple types are set to 0, enums are set to their default value.
    
   @param pMem    pointer to memory of value to be constructed
   @param pTypeDescr  type description of value to be constructed
  *)  
  procedure uno_constructData(pMem: Pointer; pTypeDescr: p_typelib_TypeDescription); cdecl; 

  (** Default constructs a value. All simple types are set to 0, enums are set to their default value.
    
   @param pMem    pointer to memory of value to be constructed
   @param pType   type of value to be constructed
  *)
  procedure uno_type_constructData(pMem: Pointer; pType: p_typelib_TypeDescriptionReference); cdecl; 
  
  (** Assigns a destination value with a source value.
    Widening conversion WITHOUT data loss is allowed (e.g., assigning a long with a short).
    Querying for demanded interface type is allowed.
    Assignment from any value to a value of type Any and vice versa is allowed.
    
   @param pDest   pointer to destination value
   @param pDestTypeDescr type description of destination value
   @param pSource   pointer to source value; if 0, then destination value will be assigned
                to default value
   @param pSourceTypeDescr type destination of source value
   @param queryInterface function called each time an interface needs to be queried;
                defaults (0) to uno
   @param acquire   function called each time an interface needs to be acquired;
                defaults (0) to uno
   @param release   function called each time an interface needs to be released;
                defaults (0) to uno
   @return true if destination has been successfully assigned
  *)
  function uno_assignData(pDest: Pointer; pDestTypeDescr: p_typelib_TypeDescription; 
                           pSource: Pointer; pSourceTypeDescr: p_typelib_TypeDescription;  
               queryInterface: uno_QueryInterfaceFunc;  acquire: uno_AcquireFunc;  
               release: uno_ReleaseFunc): sal_Bool; cdecl;

  (** Assigns a destination value with a source value.
    Widening conversion WITHOUT data loss is allowed (e.g., assigning a long with a short).
    Querying for demanded interface type is allowed.
    Assignment from any value to a value of type Any and vice versa is allowed.
    
   @param pDest   pointer to destination value
   @param pDestType  type of destination value
   @param pSource   pointer to source value; if 0, then destination value will be assigned
                to default value
   @param pSourceType  type of source value
   @param queryInterface function called each time an interface needs to be queried;
                defaults (0) to uno
   @param acquire   function called each time an interface needs to be acquired;
                defaults (0) to uno
   @param release   function called each time an interface needs to be released;
                defaults (0) to uno
   @return true if destination has been successfully assigned
  *)
  function uno_type_assignData(pDest: Pointer;  pDestType: p_typelib_TypeDescriptionReference;  
                    pSource: Pointer;  pSourceType: p_typelib_TypeDescriptionReference;  
                  queryInterface: uno_QueryInterfaceFunc;  acquire: uno_AcquireFunc;  
                  release: uno_ReleaseFunc): sal_Bool; cdecl;
  
  (** Tests whether a value of given type is assignable from given value.
    Widening conversion WITHOUT data loss is allowed (e.g., assigning a long with a short).
    Querying for demanded interface type is allowed.
    Assignment from any value to a value of type Any and vice versa is allowed.
    
   @param pAssignable      type
   @param pFrom   pointer to value
   @param pFromType  type of value
   @param queryInterface function called each time an interface needs to be queried;
                defaults (0) to uno
   @param release   function called each time an interface needs to be released;
                defaults (0) to uno
   @return true if value is destination has been successfully assigned
  *)
  function uno_type_isAssignableFromData(pAssignable: p_typelib_TypeDescriptionReference;  pFrom: Pointer;  
                      pFromType: p_typelib_TypeDescriptionReference;  queryInterface: uno_QueryInterfaceFunc;
                      release: uno_ReleaseFunc): sal_Bool; cdecl;
    
implementation

  function uno_equalData(pVal1: Pointer;  pVal1TypeDescr: p_typelib_TypeDescription;  pVal2: Pointer;  pVal2TypeDescr: p_typelib_TypeDescription;  queryInterface: uno_QueryInterfaceFunc;  release: uno_ReleaseFunc): sal_Bool; external cppu3LibraryName;
  function uno_type_equalData(pVal1: Pointer;  pVal1Type: p_typelib_TypeDescriptionReference;  pVal2: Pointer;  pVal2Type: p_typelib_TypeDescriptionReference;  queryInterface: uno_QueryInterfaceFunc;  release: uno_ReleaseFunc): sal_Bool; external cppu3LibraryName;
  procedure uno_copyData(pDest: Pointer;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;  acquire: uno_AcquireFunc); external cppu3LibraryName; 
  procedure uno_type_copyData(pDest: Pointer;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;  acquire: uno_AcquireFunc); external cppu3LibraryName; 
  procedure uno_copyAndConvertData(pDest: Pointer;  pSource: Pointer;  pTypeDescr: p_typelib_TypeDescription;  mapping: p_uno_Mapping); external cppu3LibraryName; 
  procedure uno_type_copyAndConvertData(pDest: Pointer;  pSource: Pointer;  pType: p_typelib_TypeDescriptionReference;  mapping: p_uno_Mapping); external cppu3LibraryName;
  procedure uno_destructData(pValue: Pointer;  pTypeDescr: p_typelib_TypeDescription;  release: uno_ReleaseFunc); external cppu3LibraryName;
  procedure uno_type_destructData(pValue: Pointer;  pType: p_typelib_TypeDescriptionReference;  release: uno_ReleaseFunc); external cppu3LibraryName; 
  procedure uno_constructData(pMem: Pointer;  pTypeDescr: p_typelib_TypeDescription); external cppu3LibraryName;
  procedure uno_type_constructData(pMem: Pointer;  pType: p_typelib_TypeDescriptionReference); external cppu3LibraryName; 
  function uno_assignData(pDest: Pointer;  pDestTypeDescr: p_typelib_TypeDescription;  pSource: Pointer;  pSourceTypeDescr: p_typelib_TypeDescription;  queryInterface: uno_QueryInterfaceFunc;  acquire: uno_AcquireFunc;  release: uno_ReleaseFunc): sal_Bool; external cppu3LibraryName;
  function uno_type_assignData(pDest: Pointer;  pDestType: p_typelib_TypeDescriptionReference;  pSource: Pointer;  pSourceType: p_typelib_TypeDescriptionReference;  queryInterface: uno_QueryInterfaceFunc;  acquire: uno_AcquireFunc;  release: uno_ReleaseFunc): sal_Bool; external cppu3LibraryName;
  function uno_type_isAssignableFromData(pAssignable: p_typelib_TypeDescriptionReference;  pFrom: Pointer;  pFromType: p_typelib_TypeDescriptionReference;  queryInterface: uno_QueryInterfaceFunc;  release: uno_ReleaseFunc): sal_Bool; external cppu3LibraryName;


end.
