unit uno_pascal_helper;

interface

{$I uno_common.inc}

uses
  //
  SysUtils
  //
  ,uno_pascal_uno
  ,uno_pascal_proxy
  ,uno_typelib_typedescription;

//------------------------------------------------------------------------------
  procedure Bootstrap(out xCompContext: Variant);
//------------------------------------------------------------------------------
  function CreateUNOInterface(sInterfaceName: AnsiString): Variant;
  function UNOQueryInterface(AInterface: Variant; aDestName: AnsiString): Variant;
//------------------------------------------------------------------------------
  function CreateUNOTypeByName(sTypeName: AnsiString): Variant;
  function CreateUNOTypeByTypeDescr(pType: p_typelib_TypeDescriptionReference): Variant;
//------------------------------------------------------------------------------
  procedure CreateUNOStructure(sStructureName: AnsiString; var Res: Variant);
//------------------------------------------------------------------------------
  function GetDocumentName(AUNODocument: Variant): String;
  function GetFileURLFromFileName(AName: String): String;
//------------------------------------------------------------------------------
  // MacrosURL vnd.sun.star.script:<LibName>.<ModukName>.<MacrosName>?language=<Lang>&location=application
  function ConstructMacroURL(ALibName, AModulName, AMacrosName, ALanguage: String): String;
  function RunMacros( const ADocument: Variant; MacrosURL: String;
                      MacrosInParams: Variant; var MacrosOutParams, MacrosOutIndexses: Variant): Variant;
  procedure AppEndOfVarArray(var AArray: Variant; const AValue: Variant);
//------------------------------------------------------------------------------
  function CloseDocument(const ADoc: Variant): Boolean;
//------------------------------------------------------------------------------

const
  SDefaultFileName = 'OOFile';

implementation

uses
  Variants
  ,uno_pascal_type
  ,uno_pascal_struct
  ,uno_pascal_helper_dll
  ,uno_shared
  ;

procedure Bootstrap(out xCompContext: Variant);
begin
  with TVarData(xCompContext) do begin
    VType := UNO_Interface.VarType;
    PascalHelper_BootStrap;
    VPointer := nil;
    if Assigned(Pascal_Bridge) then
      VPointer := Pointer(Pascal_Bridge.FindProxyByInterfaceName('com.sun.star.uno.XComponentContext'));
    Assert( Assigned(VPointer), sError_Bootstrap);
  end;
end;

function CloseDocument(const ADoc: Variant): Boolean;
var
  xCloseable: Variant;
begin
  Result := False;
  xCloseable := UNOQueryInterface( ADoc,'com.sun.star.util.XCloseable');
  if not VarIsNull(xCloseable)
    then xCloseable.close(true)
    else ADoc.dispose;
  Result := True;
end;

function CreateUNOInterface(sInterfaceName: AnsiString): Variant;
begin
  VarClear(Result);
  with TVarData(Result) do begin
    VType := UNO_Interface.VarType;
    VPointer := TUNO_Pascal_proxy.Create(sInterfaceName);
  end;
end;

function UNOQueryInterface(AInterface: Variant; aDestName: AnsiString): Variant;
var
  pNewTypeDesc: p_typelib_TypeDescriptionReference;
  NewType: Variant;
begin
  VarClear(Result);
  if not VarIsNull(AInterface) then begin
    pNewTypeDesc := nil;
    GetTypeDescriptionByName( p_typelib_TypeDescription(pNewTypeDesc), aDestName);
    Assert( Assigned(pNewTypeDesc), Format(sError_UNOQueryInterface_CantGetTypeDescriptionWithName,[aDestName]) );

    typelib_typedescription_complete(@pNewTypeDesc.pType);
    Assert( pNewTypeDesc^.pType^.bComplete, Format(sError_UNOQueryInterface_CantCompleteTypeDescriptionWithName,[aDestName]) );

    NewType := CreateUNOTypeByTypeDescr(pNewTypeDesc);
    Result := AInterface.QueryInterface(NewType);
  end;
// TODO: How free NewType?
//  typelib_typedescriptionreference_release(pNewTypeDesc);
end;

//------------------------------------------------------------------------------
function CreateUNOTypeByName(sTypeName: AnsiString): Variant;
begin
  with TVarData(Result) do begin
    VType := UNO_Type.VarType;
    VPointer := TUNO_Pascal_Type.CreateByName(sTypeName);
  end;
end;

function CreateUNOTypeByTypeDescr(pType: p_typelib_TypeDescriptionReference): Variant;
begin
  with TVarData(Result) do begin
    VType := UNO_Type.VarType;
    VPointer := TUNO_Pascal_Type.CreateByUNOType(pType);
  end;
end;

//------------------------------------------------------------------------------
procedure CreateUNOStructure(sStructureName: AnsiString; var Res: Variant);
begin
  VarClear(Res);
  with TVarData(Res) do begin
    VType := UNO_Structure.VarType;
    VPointer := TUNO_Pascal_Struct.CreateByName(sStructureName);
  end;
end;

//------------------------------------------------------------------------------
// Convert path to any file to uno URL
function GetFileURLFromFileName(AName: String): String;
var
  tmpFileName: String;
  i: Integer;
begin
  tmpFileName := ExpandFileName(AName);
  for i := 0 to Length(tmpFileName)-1 do begin
    if tmpFileName[i]='\' then tmpFileName[i] := '/';
//    if tmpFileName[i]=' ' then tmpFileName[i] := '%20';
  end;
  Result := 'file:///'+tmpFileName;
end;

//------------------------------------------------------------------------------
function ConstructMacroURL(ALibName, AModulName, AMacrosName, ALanguage: String): String;
begin
  Result := 'vnd.sun.star.script:'+ALibName+'.'+AModulName+'.'
          +AMacrosName+'?language='+ALanguage+'&location=application';
end;

function RunMacros( const ADocument: Variant; MacrosURL: String;
                    MacrosInParams: Variant; var MacrosOutParams,
                    MacrosOutIndexses: Variant): Variant;
var
  xScriptPS
  ,xScriptProvider
  ,xScript
  : Variant;
begin                                       
  xScriptPS := UNOQueryInterface( ADocument, 'com.sun.star.script.provider.XScriptProviderSupplier');
  Assert( not VarIsNull(xScriptPS), 'Error. Can not get "com.sun.star.script.provider.XScriptProviderSupplier" ');

  xScriptProvider := UNOQueryInterface( xScriptPS.getScriptProvider, 'com.sun.star.script.provider.XScriptProvider');
  Assert( not VarIsNull(xScriptPS), 'Error. Can not get "com.sun.star.script.provider.XScriptProvider" ');

  xScript := UNOQueryInterface( xScriptProvider.getScript( MacrosURL ),
                              'com.sun.star.script.provider.XScript');
  Assert( not VarIsNull(xScript), 'Error. Can not get vnd.sun.star.script:' + MacrosURL);
  Result := xScript.invoke( MacrosInParams, MacrosOutIndexses, MacrosOutParams );
end;
//------------------------------------------------------------------------------

procedure AppEndOfVarArray(var AArray: Variant; const AValue: Variant);
var
  HB: Integer;
begin
  if not VarIsArray(AArray)
    then AArray := VarArrayCreate([0,-1], varVariant);
  HB := VarArrayHighBound(AArray, 1);
  VarArrayRedim(AArray, HB+1);
//  AArray[HB+1] := AValue;
  VarArrayPut(AArray, AValue, [HB+1])
end;

function GetDocumentName(AUNODocument: Variant): String;
var
  sFileName: String;
  i: integer;
  tmp1: Variant;
begin
  tmp1 := AUNODocument.getArgs;
  Result := SDefaultFileName;
  if VarIsArray(tmp1) then
    for i := VarArrayLowBound(tmp1,1) to VarArrayHighBound(tmp1,1) do
      if tmp1[i].Name='Title' then
        Result := VarToStr(tmp1[i].Value);
end;

end.
