unit uno_pascal_helper_dll;

{$I uno_common.inc}

interface

uses
  uno_debug;

const
{$IFDEF MSWINDOWS}
  BridgeLibraryName = 'pas_uno.dll';
  cppuhelper3_LibraryName = 'cppuhelper3MSC.dll';
  BootstrapFunctionName = '?bootstrap@cppu@@YA?AV?$Reference@VXComponentContext@uno@star@sun@com@@@uno@star@sun@com@@XZ';
  CPPEnvironmentName = 'msci';
{$ENDIF}
{$IFDEF LINUX}
  BridgeLibraryName = 'libpas_uno.so';
  cppuhelper3_LibraryName = 'libuno_cppuhelpergcc3.so';
  BootstrapFunctionName = '_ZN4cppu9bootstrapEv';
  CPPEnvironmentName = 'gcc3';
{$ENDIF}

type
  TReference = record
    pXInterface: Pointer;
    Fake: array[Byte] of Byte;
  end;

procedure PascalHelper_BootStrap;
//procedure PascalHelper_BootStrap; stdcall;

implementation

uses
   SysUtils
  ,uno_uno_environment
  ,uno_uno_mapping
  ,uno_rtl_ustring
  ,uno_typelib_typedescription
  ,uno_dispatcher
  ,uno_shared
  ,uno_pascal_uno
  ;

type
  TInitEnvironment = procedure(var Env: uno_Environment); cdecl;
  TGetMapping = procedure(var pMapping: p_uno_Mapping; pFrom, pTo: p_uno_Environment); cdecl;

  function bootstrap: TReference; cdecl; external cppuhelper3_LibraryName name BootstrapFunctionName;
  procedure InitBridge(AInitEnvironment: TInitEnvironment; AGetMapping: TGetMapping); external BridgeLibraryName name 'InitBridge';

  procedure PascalHelper_BootStrap;
  var
    ref: TReference;

    pPascalEnv
    ,pCppEnv
    ,pUnoEnv        : p_uno_Environment;

    pPascalEnvName
    ,pCppEnvName
    ,pUnoEnvName    : prtl_uString;

    pUno2Pascal
    ,pCpp2Uno       : p_uno_Mapping;

    pTD             : p_typelib_InterfaceTypeDescription;

    pUNOXComponentContext
    ,pPascalXComponentContext : p_uno_Interface;
  begin
    InitBridge( uno_pascal_uno.uno_initEnvironment, uno_pascal_uno.uno_ext_getMapping);

    DebugDump('Bootstrap: try to bootstrap the office itself...');
    FillChar(ref, SizeOf(ref), 0);
    ref := bootstrap;
    DebugDump('Bootstrap: office was bootstrapped');

    pPascalEnv := nil;
    pCppEnv := nil;
    pUnoEnv := nil;

    pPascalEnvName := String_to_rtl_uString('pas');
    pCppEnvName := String_to_rtl_uString(CPPEnvironmentName);
    pUnoEnvName := String_to_rtl_uString('uno');

    DebugDump('Bootstrap: try to create pascal environment...');
    uno_createEnvironment( pPascalEnv, pPascalEnvName, nil );
    DebugDump('Bootstrap: pascal environment pointer: 0x'+IntToHex(Cardinal(pPascalEnv), SizeOf(Cardinal)*2));

    DebugDump('Bootstrap: try to get C++ and UNO environments...');
    uno_createEnvironment( pCppEnv, pCppEnvName, nil );
    uno_createEnvironment( pUnoEnv, pUnoEnvName, nil );
    DebugDump('Bootstrap: C++ environment pointer: 0x'+IntToHex(Cardinal(pCppEnv), SizeOf(Cardinal)*2));
    DebugDump('Bootstrap: UNO environment pointer: 0x'+IntToHex(Cardinal(pUnoEnv), SizeOf(Cardinal)*2));

    pUno2Pascal := nil;
    pCpp2Uno := nil;

    pTD := nil;
    GetTypeDescriptionByName(p_typelib_TypeDescription(pTD), 'com.sun.star.uno.XComponentContext');
    DebugDump('Bootstrap: acquired type description for '+rtl_uString_to_String(pTD^.aBase.pTypeName));

    DebugDump('Bootstrap: try to acquire ''C++-to-UNO'' mapping...');
    uno_getMapping( pCpp2Uno, pCppEnv, pUnoEnv, nil);
    DebugDump('Bootstrap: mapping was acquired');

    pUNOXComponentContext := nil;
    DebugDump('Bootstrap: try to map XComponentContext from C++ to UNO...');
    pCpp2Uno.mapInterface(pCpp2Uno, Pointer(pUNOXComponentContext), ref.pXInterface, pTD);
    DebugDump('Bootstrap: XComponentContext was mapped');

    DebugDump('Bootstrap: try to acquire ''UNO-to-Pascal'' mapping...');
    uno_getMapping(pUno2Pascal, pUnoEnv, pPascalEnv, nil);
    DebugDump('Bootstrap: mapping was acquired');

    pPascalXComponentContext := nil;
    DebugDump('Bootstrap: try to map XComponentContext from UNO to Pascal...');
    pUno2Pascal.mapInterface(pUno2Pascal, Pointer(pPascalXComponentContext), pUNOXComponentContext, pTD);
    DebugDump('Bootstrap: XComponentContext was mapped');

    typelib_typedescription_release(@pTD^.aBase);
    rtl_uString_release(pPascalEnvName);
    rtl_uString_release(pCppEnvName);
    rtl_uString_release(pUnoEnvName);
  end;

end.
