unit uno_pascal_struct;

interface

uses
  //
  Variants
  ,SysUtils
  ,Classes
  //
  ,uno_typelib_typedescription
  ,uno_debug;

type

  TtmpStructMember = record
    ParamPTR: Pointer;
    ParamOffset: Int64;
    ParamTD: p_typelib_TypeDescriptionReference;
    ParamName: AnsiString;
  end;

//------------------------------------------------------------------------------
  TUNO_Pascal_Struct = class
  protected
    pUNOStructurePTR: Pointer;
    pUNOStructureTD: p_typelib_StructTypeDescription;
    tmpStructMembersArray: array of TtmpStructMember;
    fReferencesCounter: Integer;

    fStructureSize: Integer;
    fCreatedByUNOStruct: Boolean;

    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;

    procedure CalcAbsFildsPTR;
    function ConvertMembersAndCalcStructSize: Integer;
  protected
    function FindMemberIndexByName(AName: AnsiString): Integer;

    procedure acquire; cdecl;
    procedure release; cdecl;
  public
    constructor CreateByName(aStructureName: String);
    constructor CreateByUNOStruct(pUNOStruct: Pointer; pUNOStructureTDR: p_typelib_TypeDescriptionReference);

    property ReferencesCounter: Integer read fReferencesCounter;
    property UNOStructurePTR: Pointer read pUNOStructurePTR;
    property StructureSize: Integer read fStructureSize;
    property UNOStructureTypeDescription: p_typelib_StructTypeDescription read pUNOStructureTD;
  end;
//------------------------------------------------------------------------------
  TUNO_Structure = class (TCustomVariantType)
  public
    procedure Clear(var V: TVarData); override;
    procedure Copy(var Dest: TVarData; const Source: TVarData; const Indirect: Boolean); override;

    function FixupIdent(const AText: string): string;

    procedure DispInvoke(Dest: PVarData;
                         const Source: TVarData;
                         CallDesc: PCallDesc;
                         Params: Pointer); override;

    function GetProperty(var Dest: TVarData;
                         const V: TVarData;
                         const Name: string): Boolean;

    function SetProperty(const V: TVarData;
                         const Name: string;
                         const ValuePTR: Pointer;
                         const ValueTD: Integer): Boolean;
  end;
//------------------------------------------------------------------------------

var
  UNO_Structure: TUNO_Structure;

implementation

uses
   uno_shared
  ,uno_sal_types
  ,uno_typelib_typeclass
  ,uno_rtl_ustring
  ,uno_Data
  ;

//------------------------------------------------------------------------------
function TUNO_Pascal_Struct.ConvertMembersAndCalcStructSize: Integer;
var
  tmp_ppName: pprtl_uString;
  tmp_ppMemberTD: pp_typelib_TypeDescriptionReference;
  tmp_pMemberOffsets : psal_Int32;
  i: integer;
begin
  SetLength(tmpStructMembersArray, pUNOStructureTD^.aBase.nMembers);

  tmp_ppMemberTD := pUNOStructureTD^.aBase.ppTypeRefs;
  tmp_ppName := pUNOStructureTD^.aBase.ppMemberNames;
  tmp_pMemberOffsets := pUNOStructureTD^.aBase.pMemberOffsets;

  Result := 0;

  for i := 0 to pUNOStructureTD^.aBase.nMembers-1 do begin
    tmpStructMembersArray[i].ParamTD := tmp_ppMemberTD^;
    tmpStructMembersArray[i].ParamOffset := tmp_pMemberOffsets^;

    tmpStructMembersArray[i].ParamName := rtl_uString_to_String(tmp_ppName^);

    Result := Result+AlignSize(tmp_ppMemberTD^.pType);

    inc(tmp_ppMemberTD);
    inc(tmp_ppName);
    inc(tmp_pMemberOffsets);
  end;
end;

procedure TUNO_Pascal_Struct.CalcAbsFildsPTR;
var
  i: integer;
begin
  for i := 0 to Length(tmpStructMembersArray)-1 do begin
    tmpStructMembersArray[i].ParamPTR := Pointer(Integer(pUNOStructurePTR)+tmpStructMembersArray[i].ParamOffset)
  end;
end;

constructor TUNO_Pascal_Struct.CreateByName(aStructureName: String);
begin
  pUNOStructureTD := nil;
  pUNOStructurePTR := nil;
  // getting Type Desc on our structure
  GetTypeDescriptionByName(p_typelib_TypeDescription(pUNOStructureTD), aStructureName);

  Assert( Assigned(pUNOStructureTD) and (pUNOStructureTD.aBase.aBase.eTypeClass=typelib_TypeClass_STRUCT),
          'Can not get Type Description on structure with name '+ aStructureName );

  fStructureSize := ConvertMembersAndCalcStructSize;
  GetMem(pUNOStructurePTR, fStructureSize);
  uno_constructData(pUNOStructurePTR, p_typelib_TypeDescription(pUNOStructureTD));
  CalcAbsFildsPTR;
  fCreatedByUNOStruct := False;
end;

constructor TUNO_Pascal_Struct.CreateByUNOStruct(pUNOStruct: Pointer; pUNOStructureTDR: p_typelib_TypeDescriptionReference);
begin
  typelib_typedescriptionreference_getDescription(p_typelib_TypeDescription(pUNOStructureTD), pUNOStructureTDR);
  fStructureSize := ConvertMembersAndCalcStructSize;
  GetMem(pUNOStructurePTR, fStructureSize);
  uno_type_copyData(pUNOStructurePTR, pUNOStruct, pUNOStructureTDR, nil);
  CalcAbsFildsPTR;                
  fCreatedByUNOStruct := True;
end;

procedure TUNO_Pascal_Struct.AfterConstruction;
begin
  fReferencesCounter := 1;
end;

procedure TUNO_Pascal_Struct.BeforeDestruction;
var
  i: integer;
begin
  uno_destructData(pUNOStructurePTR, p_typelib_TypeDescription(pUNOStructureTD), nil);
{
  if not fCreatedByUNOStruct and (fReferencesCounter = 1) then
    for i := 0 to Length(tmpStructMembersArray)-1 do
      DestroyData_and_FreeMem( tmpStructMembersArray[i].ParamPTR,
                               tmpStructMembersArray[i].ParamTD );
}                               
  FreeMem(pUNOStructurePTR);
  typelib_typedescriptionreference_release(p_typelib_TypeDescriptionReference(pUNOStructureTD));
end;

function TUNO_Pascal_Struct.FindMemberIndexByName(AName: AnsiString): Integer;
var
  i: Integer;
begin
  Result := -1;
  i := 0;
  while (i<(Length(tmpStructMembersArray))) and (Result=-1) do begin
    if (CompareText(tmpStructMembersArray[i].ParamName, AName)=0) then
      Result := i;
    inc(i);
  end;
end;

procedure TUNO_Pascal_Struct.acquire; cdecl;
begin
  Inc(fReferencesCounter);
end;

procedure TUNO_Pascal_Struct.release; cdecl;
begin
  Dec(fReferencesCounter);
end;

//------------------------------------------------------------------------------

procedure TUNO_Structure.Clear(var V: TVarData);
begin
  with V do begin
    if (TUNO_Pascal_Struct(VPointer).ReferencesCounter = 1) then
      FreeAndNil(TUNO_Pascal_Struct(VPointer))
    else begin
      TUNO_Pascal_Struct(VPointer).release;
      VPointer := nil;
    end;
    VType := varEmpty;
  end;
end;

procedure TUNO_Structure.Copy(var Dest: TVarData; const Source: TVarData; const Indirect: Boolean);
begin
  if Indirect and VarDataIsByRef(Source) then
    VarDataCopyNoInd(Dest, Source)
  else with Dest do begin
    VType := Source.VType;
    Dest.VPointer := Source.VPointer;
    TUNO_Pascal_Struct(Source.VPointer).acquire;
  end;
end;

function TUNO_Structure.GetProperty(var Dest: TVarData; const V: TVarData; const Name: string): Boolean;
var
  // bibbs
  tmpPtr: Pointer;
  tmpIndex: Integer;
  tmpUNO2Pascal: Pointer;
begin
  Result := False;
  with TUNO_Pascal_Struct(V.VPointer) do begin
    tmpIndex := FindMemberIndexByName(Name);
    if (tmpIndex>=0) then begin
      DebugDump('TUNO_Structure: getted property '+Name);

      if NeedPassByRef(tmpStructMembersArray[tmpIndex].ParamTD)
        then tmpUNO2Pascal := Pointer(tmpStructMembersArray[tmpIndex].ParamPTR^)
        else tmpUNO2Pascal := tmpStructMembersArray[tmpIndex].ParamPTR;

      UnoParam2PascalParam( tmpUNO2Pascal,
                            p_typelib_TypeDescriptionReference(tmpStructMembersArray[tmpIndex].ParamTD),
                            tmpPtr,
                            Variant(Dest));
      Result := True;
    end;
  end;
end;

function TUNO_Structure.FixupIdent(const AText: string): string;
begin
  Result := Uppercase(AText);
end;

procedure TUNO_Structure.DispInvoke(Dest: PVarData; const Source: TVarData;
  CallDesc: PCallDesc; Params: Pointer);
type
  PParamRec = ^TParamRec;
  TParamRec = array[0..3] of LongInt;

const
  CDoMethod    = $01;
  CPropertyGet = $02;
  CPropertySet = $04;

  CArgTypeMask    = $7F;
  CArgByRef       = $80;

var
  LArguments: TVarDataArray;
  LStrCount: Integer;
var
  I, LArgCount: Integer;
  LIdent: string;

  LArgType: Integer;
  LArgByRef: Boolean;
  LParamPtr: Pointer;

begin
  // Grab the identifier
  LArgCount := CallDesc^.ArgCount;
  LIdent := FixupIdent(String(PChar(@CallDesc^.ArgTypes[LArgCount])));
  // What type of invoke is this?
  case CallDesc^.CallType of
    CDoMethod: begin
      // procedure with N arguments
      Assert( Assigned(Dest), sError_ErrorInParams);
      // property get or function with 0 argument
      if ((LArgCount = 0) and (not GetProperty(Dest^, Source, LIdent))) then
          RaiseDispError;
    end;
    CPropertyGet:
      if not ((Dest <> nil) and (LArgCount = 0) and     // there must be a dest
               GetProperty(Dest^, Source, LIdent))      // only no args get op be valid
      then
        RaiseDispError;

    CPropertySet:
      if ((Dest = nil) and (LArgCount = 1)) then begin
        LArgType := CallDesc^.ArgTypes[0] and CArgTypeMask;
        // Arg by ref
        if ((CallDesc^.ArgTypes[0] and CArgByRef) <> 0)
          then LParamPtr := Pointer(Params^)
          else LParamPtr := Params;

        SetProperty(Source, LIdent, LParamPtr, LArgType);
      end else
        RaiseDispError;
    else
      RaiseDispError;
  end;
end;

function TUNO_Structure.SetProperty(const V: TVarData; const Name: string; const ValuePTR: Pointer; const ValueTD: Integer): Boolean;
var
  // bibbs
  tmpPtr: Pointer;
  tmpIndex: Integer;
  tmpUNOSize: Integer;
begin
  Result := False;
  with TUNO_Pascal_Struct(V.VPointer) do begin
    tmpIndex := FindMemberIndexByName(Name);
    if (tmpIndex>=0) then begin
      DebugDump('TUNO_Structure: setted property '+Name);
      tmpUNOSize := 0;
      tmpPtr := tmpStructMembersArray[tmpIndex].ParamPTR;
      if NeedPassByRef(tmpStructMembersArray[tmpIndex].ParamTD)
        then PascalParam2UNOParam( ValuePTR, ValueTD,
                                   tmpStructMembersArray[tmpIndex].ParamTD,
                                   PPointer(tmpPtr)^, tmpUNOSize)
        else PascalParam2UNOParam( ValuePTR, ValueTD,
                                   tmpStructMembersArray[tmpIndex].ParamTD,
                                   tmpPtr, tmpUNOSize);
      Result := True;
    end;
  end;

end;

initialization
  UNO_Structure := TUNO_Structure.Create;

finalization
  if Assigned(UNO_Structure) then
    UNO_Structure.Free;

end.
