unit uno_pascal_type;

interface

{$I uno_common.inc}

uses
  //
  Variants
  ,SysUtils
  ,Classes
  //
  ,uno_typelib_typedescription;

type

//------------------------------------------------------------------------------
  TUNO_Pascal_Type = class
  protected
    pUNOTypeTDR: p_typelib_TypeDescriptionReference;
    fReferencesCounter: Integer;

    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;

    procedure acquire; cdecl;
    procedure release; cdecl;
  public
    constructor CreateByName(aTypeName: String);
    constructor CreateByUNOType(pType: p_typelib_TypeDescriptionReference);

    property ReferencesCounter: Integer read fReferencesCounter;
    property UNOType: p_typelib_TypeDescriptionReference read pUNOTypeTDR;
  end;
//------------------------------------------------------------------------------
  TUNO_Type = class (TCustomVariantType)
  protected
    procedure Clear(var V: TVarData); override;
    procedure Copy(var Dest: TVarData; const Source: TVarData; const Indirect: Boolean); override;
  end;
//------------------------------------------------------------------------------

var
  UNO_Type: TUNO_Type;

implementation

uses
   uno_shared
  ,uno_sal_types
  ,uno_rtl_ustring
  ;

//------------------------------------------------------------------------------
constructor TUNO_Pascal_Type.CreateByName(aTypeName: String);
begin
  pUNOTypeTDR := nil;
  GetTypeDescriptionByName(p_typelib_TypeDescription(pUNOTypeTDR), aTypeName);
  Assert(Assigned(pUNOTypeTDR), Format(sError_OOType_CantGetTypeDescription, [aTypeName]));
end;

constructor TUNO_Pascal_Type.CreateByUNOType(pType: p_typelib_TypeDescriptionReference);
begin
  pUNOTypeTDR := pType;
end;

procedure TUNO_Pascal_Type.AfterConstruction;
begin
  fReferencesCounter := 1;
end;

procedure TUNO_Pascal_Type.BeforeDestruction;
begin
  typelib_typedescriptionreference_release(pUNOTypeTDR);
end;

procedure TUNO_Pascal_Type.acquire; cdecl;
begin
  Inc(fReferencesCounter);
  typelib_typedescriptionreference_acquire(pUNOTypeTDR);
end;

procedure TUNO_Pascal_Type.release; cdecl;
begin
  Dec(fReferencesCounter);
  typelib_typedescriptionreference_release(pUNOTypeTDR);
end;

//------------------------------------------------------------------------------

procedure TUNO_Type.Clear(var V: TVarData);
begin
  with V do begin
    if (TUNO_Pascal_Type(VPointer).ReferencesCounter = 1) then
      FreeAndNil(TUNO_Pascal_Type(VPointer))
    else begin
      TUNO_Pascal_Type(VPointer).release;
      VPointer := nil;
    end;
    VType := varEmpty;
  end;
end;

procedure TUNO_Type.Copy(var Dest: TVarData; const Source: TVarData; const Indirect: Boolean);
begin
  if Indirect and VarDataIsByRef(Source) then
    VarDataCopyNoInd(Dest, Source)
  else with Dest do begin
    VType := Source.VType;
    Dest.VPointer := Source.VPointer;
    TUNO_Pascal_Type(Source.VPointer).acquire;
  end;
end;

initialization
  UNO_Type := TUNO_Type.Create;

finalization
  if Assigned(UNO_Type) then begin
    UNO_Type.Free;
//    FreeAndNil(UNO_Type);
  end;

end.
