unit uno_pascal_uno;

{$I uno_common.inc}

interface

uses
   uno_uno_environment
  ,uno_uno_mapping
  ,uno_rtl_ustring
  ,uno_pascal_proxy
  ,uno_typelib_typedescription
  ,uno_shared
  ,uno_Any2
  ,SysUtils
  ;

type
//------------------------------------------------------------------------------
  //!!! indexes from 1
  TProxyList = class( TObjectsList )
  private
    function GetProxy(Index: Integer): TUNO_Pascal_proxy;
  public
    procedure BeforeDestruction; override;
    property Proxys[Index: Integer]: TUNO_Pascal_proxy read GetProxy; default;
  end;
//------------------------------------------------------------------------------

  uno_exception = record
    Mess: prtl_uString;
    Context: Pointer;
  end;
  p_uno_exception = ^uno_exception;
  pp_uno_exception = ^p_uno_exception;

  TUNO_Exception = class(Exception)
    constructor Create(Mes: String); overload;
  end;

//------------------------------------------------------------------------------
  TUNO_Pascal_Bridge = class
  private
    ProxyArray: TProxyList;
  public
    procedure AfterConstruction; override;
    procedure BeforeDestruction; override;

    function FindProxyByUnoInterface(AInterface: Pointer): Pointer;
    function FindProxyByInterfaceName(AIntName: AnsiString): TUNO_Pascal_proxy;
    function GetProxyByUnoInterface( AInterface: Pointer;
                                     InterfaceTypeDescr: p_typelib_InterfaceTypeDescription): Pointer;

    function GetUnoInterfaceByName(AInterfaceNeededToMap: Pointer; pInterfaceName: prtl_uString): Pointer;
    function GetUnoInterfaceByTypeDescription(AInterfaceNeededToMap: Pointer; pTypeDescr: p_typelib_InterfaceTypeDescription): Pointer;

    procedure RegNewProxy(AProxy: TUNO_Pascal_proxy);
    procedure UnRegProxy(AProxy: TUNO_Pascal_proxy);
  end;

//------------------------------------------------------------------------------
  procedure acquireInterface(pEnv: p_uno_ExtEnvironment; pInterface: Pointer); cdecl;
  procedure releaseInterface(pEnv: p_uno_ExtEnvironment; pInterface: Pointer); cdecl;
  procedure MapPascalToUno( pMapping: p_uno_Mapping; var pOut: Pointer; pInterface: Pointer;
                            pInterfaceTypeDescr: p_typelib_InterfaceTypeDescription ); cdecl;
  procedure MapUnoToPascal( pMapping: p_uno_Mapping; var pOut: Pointer; pInterface: Pointer;
                            pInterfaceTypeDescr: p_typelib_InterfaceTypeDescription ); cdecl;
//------------------------------------------------------------------------------


procedure computeObjectIdentifier( pEnv: p_uno_ExtEnvironment; var pOId: Prtl_uString; pInterface: Pointer ); cdecl;

function getPascalToUNO_Mapping: p_uno_Mapping;
function getUNOToPascal_Mapping: p_uno_Mapping;

function getPascal_Env: p_uno_Environment;
function getUNO_Env: p_uno_Environment;

//==============================================================================
//                          from Pascal_uno.dll
//==============================================================================
procedure uno_initEnvironment(var Env: uno_Environment); cdecl;
procedure uno_ext_getMapping(var pMapping: p_uno_Mapping; pFrom, pTo: p_uno_Environment); cdecl;
//==============================================================================

var
  Pascal_Bridge: TUNO_Pascal_Bridge;
  fPascalToUNO_Mapping, fUNOToPascal_Mapping: uno_Mapping;
  fPascal_Env, fUNO_Env                      : p_uno_Environment;

const
  CURRENT_LANGUAGE_BINDING_NAME : string = 'pas';
  UNO_LB_UNO                    : string = 'uno';

implementation

uses
  uno_debug,
  uno_sal_types;

//==============================================================================
//                                 TProxyList
//==============================================================================
procedure TProxyList.BeforeDestruction;
var
  i: integer;
begin
  while count > 0 do Proxys[1].Free;
  inherited BeforeDestruction;
end;

function TProxyList.GetProxy(Index: Integer): TUNO_Pascal_proxy;
begin
  CheckIndex(Index);
  Result := TUNO_Pascal_proxy(fObjects[Index-1]);
end;
//------------------------------------------------------------------------------
constructor TUNO_Exception.create(Mes: String);
begin
  Message := Mes;
end;

//==============================================================================
//                                 TUNO_Pascal_Bridge
//==============================================================================
procedure TUNO_Pascal_Bridge.AfterConstruction;
begin
  ProxyArray := TProxyList.Create;
end;

procedure TUNO_Pascal_Bridge.BeforeDestruction;
begin
  ProxyArray.Free;
end;

function TUNO_Pascal_Bridge.FindProxyByUnoInterface(AInterface: Pointer): Pointer;
var
  i: Integer;
begin
  Result := nil;
  i := 1;
  while ( Result=nil ) and ( i<=ProxyArray.Count ) do begin
    if (ProxyArray[i]=AInterface) then
      Result := AInterface;
    inc(i);
  end;
end;

function TUNO_Pascal_Bridge.FindProxyByInterfaceName(AIntName: AnsiString): TUNO_Pascal_proxy;
var
  i: Integer;
begin
  Result := nil;
  i := 1;
  while ( Result=nil ) and ( i<=ProxyArray.Count ) do begin
    if (CompareText(ProxyArray[i].ProxyName,AIntName)=0) then
      Result := ProxyArray[i];
    inc(i);
  end;
end;

function TUNO_Pascal_Bridge.GetProxyByUnoInterface(AInterface: Pointer;
                 InterfaceTypeDescr: p_typelib_InterfaceTypeDescription): Pointer;
begin
  Result := FindProxyByUnoInterface(AInterface);
  if not Assigned(Result)
    then Result := TUNO_Pascal_proxy.CreateByUNOInterface( AInterface,
                                                           InterfaceTypeDescr );
end;

function TUNO_Pascal_Bridge.GetUnoInterfaceByName(AInterfaceNeededToMap: Pointer;
                                         pInterfaceName: prtl_uString): Pointer;
var
  ATypeDescr: p_typelib_InterfaceTypeDescription;
begin
  //TODO: add exceptions
  ATypeDescr := nil;
  Result := nil;
  typelib_typedescription_getByName(p_typelib_TypeDescription(ATypeDescr), pInterfaceName);

  if Assigned(ATypeDescr) then
    Result := GetUnoInterfaceByTypeDescription(AInterfaceNeededToMap, ATypeDescr);
end;

function TUNO_Pascal_Bridge.GetUnoInterfaceByTypeDescription(AInterfaceNeededToMap: Pointer;
                                  pTypeDescr: p_typelib_InterfaceTypeDescription): Pointer;
var
  pOut: Pointer;
begin
  pOut := nil;
  if (getUnoToPascal_Mapping<>nil) then
    fUnoToPascal_Mapping.mapInterface( @fUnoToPascal_Mapping, pOut, AInterfaceNeededToMap, pTypeDescr );
  if Assigned(pOut) then
    Result := pOut
  else
    Result := nil;
end;

procedure TUNO_Pascal_Bridge.RegNewProxy(AProxy: TUNO_Pascal_proxy);
begin
  ProxyArray.Add(AProxy);
end;

procedure TUNO_Pascal_Bridge.UnRegProxy(AProxy: TUNO_Pascal_proxy);
begin
  ProxyArray.Remove(AProxy);
end;

//------------------------------------------------------------------------------
procedure computeObjectIdentifier( pEnv: p_uno_ExtEnvironment;
                            var pOId: Prtl_uString; pInterface: Pointer ); cdecl;
begin
  pOID := String_to_rtl_uString(IntToStr(Integer(pInterface)));
end;

function getPascalToUNO_Mapping: p_uno_Mapping;
var
  tmpPtr: Pointer;
begin
  tmpPtr := @fPascalToUNO_Mapping;
  if Assigned(tmpPtr) then Result := tmpPtr
  else begin
    if (getPascal_Env<>nil) and (getUNO_Env<>nil) then begin
      uno_getMapping(p_uno_mapping(tmpPtr), getPascal_Env, getUNO_Env, nil);
      Result := nil;
      Assert(Assigned(tmpPtr), sError_CantGetPascalMapping);
      Result := tmpPtr;
    end;
  end;
end;

function getUNOToPascal_Mapping: p_uno_Mapping;
var
  tmpPtr: Pointer;
begin
  tmpPtr := @fUNOToPascal_Mapping;
  if Assigned(tmpPtr) then Result := tmpPtr
  else begin
    if (getPascal_Env<>nil) and (getUNO_Env<>nil) then begin
      uno_getMapping(p_uno_mapping(tmpPtr), getUNO_Env, getPascal_Env, nil);
      Result := nil;
      Assert(Assigned(tmpPtr), sError_CantGetUnoMapping);
      Result := tmpPtr;
    end;
  end;
end;

function getPascal_Env: p_uno_Environment;
begin
  if Assigned(fPascal_Env) then
    Result := fPascal_Env
  else
    Assert(Assigned(fPascal_Env),sError_PascalEnvFailed);
end;

function getUNO_Env: p_uno_Environment;
var
  pEnvName: prtl_uString;
begin
  if Assigned(fUNO_Env) then
    Result := fUNO_Env
  else begin
    // getting Uno Enviroment
    pEnvName := String_to_rtl_uString(UNO_LB_UNO);
    uno_getEnvironment(fUno_Env, pEnvName, nil);
    rtl_uString_release(pEnvName);
    Result := nil;
    Assert(Assigned(fUno_Env), sError_UnoEnvFailed);
    Result := fUno_Env;
  end;
end;

//------------------------------------------------------------------------------

procedure thisDisposing( pEnv: p_uno_Environment ); cdecl;
begin
  pEnv^.pContext := nil;
  if Assigned(Pascal_Bridge) then
    Pascal_Bridge.Free;
end;

procedure acquireInterface(pEnv: p_uno_ExtEnvironment; pInterface: Pointer); cdecl;
begin
   TUNO_Pascal_proxy( pInterface ).acquireInterface;
end;

procedure releaseInterface(pEnv: p_uno_ExtEnvironment; pInterface: Pointer); cdecl;
begin
   TUNO_Pascal_proxy( pInterface ).releaseInterface;
end;

procedure MapPascalToUno( pMapping: p_uno_Mapping; var pOut: Pointer; pInterface: Pointer;
                       pInterfaceTypeDescr: p_typelib_InterfaceTypeDescription ); cdecl;
begin
  // TODO: add exceptions
  pOut := nil;
  if Assigned(Pascal_Bridge) then begin
    pOut := TUNO_Pascal_proxy(pInterface).UNOInterface;
    if not Assigned(pOut) then begin
      pOut := Pascal_Bridge.GetUnoInterfaceByTypeDescription(pInterface, pInterfaceTypeDescr);
    end;
  end;
end;

procedure MapUnoToPascal( pMapping: p_uno_Mapping; var pOut: Pointer; pInterface: Pointer;
                       pInterfaceTypeDescr: p_typelib_InterfaceTypeDescription ); cdecl;
begin
  // TODO: add exception
  pOut := nil;
  if Assigned(Pascal_Bridge) then
    pOut := Pascal_Bridge.GetProxyByUnoInterface(pInterface, pInterfaceTypeDescr);
end;

procedure freeMapping(pMapping: p_uno_Mapping); cdecl;
begin
   //TODO: make true variant;
   pMapping := nil;
end;

procedure MappingAcquire( pMapping: p_uno_Mapping ); cdecl;
begin
   // bibb;
end;

procedure MappingRelease( pMapping: p_uno_Mapping ); cdecl;
begin
   // bibb;
end;

// -----------------------------------------------------------------------------

procedure uno_initEnvironment(var Env: uno_Environment); cdecl;
begin
  Env.pExtEnv^.computeObjectIdentifier := computeObjectIdentifier;
  Env.pExtEnv^.acquireInterface := acquireInterface;
  Env.pExtEnv^.releaseInterface := releaseInterface;
{
  Env.acquire :=
  Env.release :=
  Env.acquireWeak :=
  Env.releaseWeak :=
  Env.harden :=
}
  Env.dispose := thisDisposing;

  Pascal_Bridge := TUNO_Pascal_Bridge.Create;
  fPascal_Env := @Env;

  DebugDump('bridge: initEnvironment passed');
end;

procedure uno_ext_getMapping(var pMapping: p_uno_Mapping; pFrom, pTo: p_uno_Environment); cdecl;
var
  tmpMapping: uno_Mapping;
  tmpPtr: p_uno_Mapping;
  tmpStr: AnsiString;
  tmp: TUNO_Pascal_proxy;
begin
  if ( Assigned(pFrom) and Assigned(pTo) and
       Assigned(pFrom.pExtEnv) and Assigned(pTo.pExtEnv) ) then
  begin
    tmpMapping.acquire := MappingAcquire;
    tmpMapping.release := MappingRelease;

    if ( 0 = rtl_ustr_ascii_compare(@pFrom^.pTypeName^.buffer,
                                    psal_Char(CURRENT_LANGUAGE_BINDING_NAME) ) )
    and
       ( 0 = rtl_ustr_ascii_compare( @pTo^.pTypeName^.buffer,
                                     psal_Char(UNO_LB_UNO) ) )
    then begin
      // ref count initially 1
      tmpMapping.mapInterface := MapPascalToUNO;
      fPascalToUNO_Mapping := tmpMapping;
      tmpPtr := @fPascalToUNO_Mapping;
    end else if ( 0 = rtl_ustr_ascii_compare( @pTo^.pTypeName^.buffer,
                                    Psal_Char(CURRENT_LANGUAGE_BINDING_NAME) ) )
    and
                ( 0 = rtl_ustr_ascii_compare( @pFrom^.pTypeName^.buffer,
                                                       Psal_Char(UNO_LB_UNO) ) )
    then begin
      // ref count initially 1
      tmpMapping.mapInterface := MapUNOToPascal;
      fUNOToPascal_Mapping := tmpMapping;
      tmpPtr := @fUNOToPascal_Mapping;
    end;
    //TODO: getting from . for what?
    uno_registerMapping( tmpPtr, freeMapping, pFrom, pTo, 0 );
    pMapping := tmpPtr;

    DebugDump('bridge: ext_getMapping passed');
  end;
end;

initialization

finalization
  if Assigned(Pascal_Bridge) then
    Pascal_Bridge.Free;

end.


