unit uno_rtl_string;

interface

{$I uno_common.inc}

uses
   uno_sal_config
  ,uno_sal_types
  ,uno_rtl_textcvt
  ,uno_rtl_textenc;

type
  _rtl_String = record
    refCount : sal_Int32;
    length : sal_Int32;
    buffer : array[0..0] of sal_Char;
  end;
  rtl_String = _rtl_String;

Type
  Prtl_String  = ^rtl_String;
  PPrtl_String  = ^Prtl_String;

  { =======================================================================  }
  {* Return the length of a string.

      The length is equal to the number of 8-bit characters in the string,
      without the terminating NUL character.

      @param str
      a null-terminated string.

      @return
      the length of the sequence of characters represented by this string,
      excluding the terminating NUL character.
    }
  function rtl_str_getLength(str:Psal_Char):sal_Int32; cdecl;

  {* Compare two strings.

      The comparison is based on the numeric value of each character in the
      strings and returns a value indicating their relationship.  This function
      cannot be used for language-specific sorting.  Both strings must be
      null-terminated.

      @param first
      the first null-terminated string to be compared.

      @param second
      the second null-terminated string which is compared with the first one.
  
      @return
      0 if both strings are equal, a value less than 0 if the first string is
      less than the second string, and a value greater than 0 if the first
      string is greater than the second string.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_compare(first:Psal_Char; second:Psal_Char):sal_Int32; cdecl;

  {* Compare two strings.
  
      The comparison is based on the numeric value of each character in the
      strings and returns a value indicating their relationship.  This function
      cannot be used for language-specific sorting.
  
      @param first
      the first string to be compared.  Need not be null-terminated, but must be
      at least as long as the specified firstLen.
  
      @param firstLen
      the length of the first string.
  
      @param second
      the second string which is compared with the first one.  Need not be
      null-terminated, but must be at least as long as the specified secondLen.
  
      @param secondLen
      the length of the second string.
  
      @return
      0 if both strings are equal, a value less than 0 if the first string is
      less than the second string, and a value greater than 0 if the first
      string is greater than the second string.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_compare_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32):sal_Int32; cdecl;

  {* Compare two strings with a maximum count of characters.
  
      The comparison is based on the numeric value of each character in the
      strings and returns a value indicating their relationship.  This function
      cannot be used for language-specific sorting.
  
      @param first
      the first string to be compared.  Need not be null-terminated, but must be
      at least as long as the specified firstLen.
  
      @param firstLen
      the length of the first string.
  
      @param second
      the second string which is compared with the first one.  Need not be
      null-terminated, but must be at least as long as the specified secondLen.
  
      @param secondLen
      the length of the second string.
  
      @param shortenedLen
      the maximum number of characters to compare.  This length can be greater
      or smaller than the lengths of the two strings.
  
      @return
      0 if both substrings are equal, a value less than 0 if the first substring
      is less than the second substring, and a value greater than 0 if the first
      substring is greater than the second substring.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_shortenedCompare_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32; shortenedLen:sal_Int32):sal_Int32; cdecl;

  {* Compare two strings from back to front.

      The comparison is based on the numeric value of each character in the
      strings and returns a value indicating their relationship.  This function
      cannot be used for language-specific sorting.

      @param first
      the first string to be compared.  Need not be null-terminated, but must be
      at least as long as the specified firstLen.

      @param firstLen
      the length of the first string.

      @param second
      the second string which is compared with the first one.  Need not be
      null-terminated, but must be at least as long as the specified secondLen.

      @param secondLen
      the length of the second string.

      @return
      0 if both strings are equal, a value less than 0 if the first string
      compares less than the second string, and a value greater than 0 if the
      first string compares greater than the second string.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_reverseCompare_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32):sal_Int32; cdecl;

  {* Compare two strings, ignoring the case of ASCII characters.

      The comparison is based on the numeric value of each character in the
      strings and returns a value indicating their relationship.  Character
      values between 65 and 90 (ASCII A--Z) are interpreted as values between 97
      and 122 (ASCII a--z).  This function cannot be used for language-specific
      sorting.  Both strings must be null-terminated.

      @param first
      the first null-terminated string to be compared.

      @param second
      the second null-terminated string which is compared with the first one.

      @return
      0 if both strings are equal, a value less than 0 if the first string is
      less than the second string, and a value greater than 0 if the first
      string is greater than the second string.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_compareIgnoreAsciiCase(first:Psal_Char; second:Psal_Char):sal_Int32; cdecl;

  {* Compare two strings, ignoring the case of ASCII characters.

      The comparison is based on the numeric value of each character in the
      strings and returns a value indicating their relationship.  Character
      values between 65 and 90 (ASCII A--Z) are interpreted as values between 97
      and 122 (ASCII a--z).  This function cannot be used for language-specific
      sorting.

      @param first
      the first string to be compared.  Need not be null-terminated, but must be
      at least as long as the specified firstLen.

      @param firstLen
      the length of the first string.

      @param second
      the second string which is compared with the first one.  Need not be
      null-terminated, but must be at least as long as the specified secondLen.

      @param secondLen
      the length of the second string.

      @return
      0 if both strings are equal, a value less than 0 if the first string is
      less than the second string, and a value greater than 0 if the first
      string is greater than the second string.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_compareIgnoreAsciiCase_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32):sal_Int32; cdecl;

  {* Compare two strings with a maximum count of characters, ignoring the case
      of ASCII characters.

      The comparison is based on the numeric value of each character in the
      strings and returns a value indicating their relationship.  Character
      values between 65 and 90 (ASCII A--Z) are interpreted as values between 97
      and 122 (ASCII a--z).  This function cannot be used for language-specific
      sorting.

      @param first
      the first string to be compared.  Need not be null-terminated, but must be
      at least as long as the specified firstLen.

      @param firstLen
      the length of the first string.

      @param second
      the second string which is compared with the first one.  Need not be
      null-terminated, but must be at least as long as the specified secondLen.

      @param secondLen
      the length of the second string.

      @param shortenedLen
      the maximum number of characters to compare.  This length can be greater
      or smaller than the lengths of the two strings.

      @return
      0 if both substrings are equal, a value less than 0 if the first substring
      is less than the second substring, and a value greater than 0 if the first
      substring is greater than the second substring.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_shortenedCompareIgnoreAsciiCase_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32; shortenedLen:sal_Int32):sal_Int32; cdecl;

  {* Return a hash code for a string.

      It is not allowed to store the hash code persistently, because later
      versions could return other hash codes.  The string must be
      null-terminated.
  
      @param str
      a null-terminated string.
  
      @return
      a hash code for the given string.
    }
(* Const before type ignored *)
  function rtl_str_hashCode(str:Psal_Char):sal_Int32; cdecl;

  {* Return a hash code for a string.
  
      It is not allowed to store the hash code persistently, because later
      versions could return other hash codes.

      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.
  
      @param len
      the length of the string.
  
      @return
      a hash code for the given string.
    }
(* Const before type ignored *)
  function rtl_str_hashCode_WithLength(str:Psal_Char; len:sal_Int32):sal_Int32; cdecl;

  {* Search for the first occurrence of a character within a string.
  
      The string must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @param ch
      the character to be searched for.
  
      @return
      the index (starting at 0) of the first occurrence of the character in the
      string, or -1 if the character does not occur.
    }
(* Const before type ignored *)
  function rtl_str_indexOfChar(str:Psal_Char; ch:sal_Char):sal_Int32; cdecl;

  {* Search for the first occurrence of a character within a string.
  
      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.
  
      @param len
      the length of the string.
  
      @param ch
      the character to be searched for.
  
      @return
      the index (starting at 0) of the first occurrence of the character in the
      string, or -1 if the character does not occur.
    }
(* Const before type ignored *)
  function rtl_str_indexOfChar_WithLength(str:Psal_Char; len:sal_Int32; ch:sal_Char):sal_Int32; cdecl;

  {* Search for the last occurrence of a character within a string.
  
      The string must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @param ch
      the character to be searched for.
  
      @return
      the index (starting at 0) of the last occurrence of the character in the
      string, or -1 if the character does not occur.  The returned value is
      always smaller than the string length.
    }
(* Const before type ignored *)
  function rtl_str_lastIndexOfChar(str:Psal_Char; ch:sal_Char):sal_Int32; cdecl;

  {* Search for the last occurrence of a character within a string.
  
      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.
  
      @param len
      the length of the string.
  
      @param ch
      the character to be searched for.
  
      @return
      the index (starting at 0) of the last occurrence of the character in the
      string, or -1 if the character does not occur.  The returned value is
      always smaller than the string length.
    }
(* Const before type ignored *)
  function rtl_str_lastIndexOfChar_WithLength(str:Psal_Char; len:sal_Int32; ch:sal_Char):sal_Int32; cdecl;

  {* Search for the first occurrence of a substring within a string.
  
      If subStr is empty, or both str and subStr are empty, -1 is returned.
      Both strings must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @param subStr
      the null-terminated substring to be searched for.
  
      @return
      the index (starting at 0) of the first character of the first occurrence
      of the substring within the string, or -1 if the substring does not occur.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_indexOfStr(str:Psal_Char; subStr:Psal_Char):sal_Int32; cdecl;

  {* Search for the first occurrence of a substring within a string.
  
      If subStr is empty, or both str and subStr are empty, -1 is returned.

      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.
  
      @param len
      the length of the string.
  
      @param subStr
      the substring to be searched for.  Need not be null-terminated, but must
      be at least as long as the specified subLen.
  
      @param subLen
      the length of the substring.
  
      @return
      the index (starting at 0) of the first character of the first occurrence
      of the substring within the string, or -1 if the substring does not occur.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_indexOfStr_WithLength(str:Psal_Char; len:sal_Int32; subStr:Psal_Char; subLen:sal_Int32):sal_Int32; cdecl;

  {* Search for the last occurrence of a substring within a string.
  
      If subStr is empty, or both str and subStr are empty, -1 is returned.
      Both strings must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @param subStr
      the null-terminated substring to be searched for.
  
      @return
      the index (starting at 0) of the first character of the last occurrence
      of the substring within the string, or -1 if the substring does not occur.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_lastIndexOfStr(str:Psal_Char; subStr:Psal_Char):sal_Int32; cdecl;

  {* Search for the last occurrence of a substring within a string.
  
      If subStr is empty, or both str and subStr are empty, -1 is returned.
  
      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.
  
      @param len
      the length of the string.
  
      @param subStr
      the substring to be searched for.  Need not be null-terminated, but must
      be at least as long as the specified subLen.
  
      @param subLen
      the length of the substring.
  
      @return
      the index (starting at 0) of the first character of the first occurrence
      of the substring within the string, or -1 if the substring does not occur.
    }
(* Const before type ignored *)
(* Const before type ignored *)
  function rtl_str_lastIndexOfStr_WithLength(str:Psal_Char; len:sal_Int32; subStr:Psal_Char; subLen:sal_Int32):sal_Int32; cdecl;

  {* Replace all occurrences of a single character within a string.
  
      If oldChar does not occur within str, then the string is not modified.
      The string must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @param oldChar
      the old character.
  
      @param newChar
      the new character.
    }
  procedure rtl_str_replaceChar(str:Psal_Char; oldChar:sal_Char; newChar:sal_Char); cdecl;

  {* Replace all occurrences of a single character within a string.
  
      If oldChar does not occur within str, then the string is not modified.
  
      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.
  
      @param len
      the length of the string.
  
      @param oldChar
      the old character.
  
      @param newChar
      the new character.
    }
  procedure rtl_str_replaceChar_WithLength(str:Psal_Char; len:sal_Int32; oldChar:sal_Char; newChar:sal_Char); cdecl;

  {* Convert all ASCII uppercase letters to lowercase within a string.
  
      The characters with values between 65 and 90 (ASCII A--Z) are replaced
      with values between 97 and 122 (ASCII a--z).  The string must be
      null-terminated.
  
      @param str
      a null-terminated string.
    }
  procedure rtl_str_toAsciiLowerCase(str:Psal_Char); cdecl;

  {* Convert all ASCII uppercase letters to lowercase within a string.
  
      The characters with values between 65 and 90 (ASCII A--Z) are replaced
      with values between 97 and 122 (ASCII a--z).
  
      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.
  
      @param len
      the length of the string.
    }
  procedure rtl_str_toAsciiLowerCase_WithLength(str:Psal_Char; len:sal_Int32); cdecl;

  {* Convert all ASCII lowercase letters to uppercase within a string.
  
      The characters with values between 97 and 122 (ASCII a--z) are replaced
      with values between 65 and 90 (ASCII A--Z).  The string must be
      null-terminated.
  
      @param str
      a null-terminated string.
    }
  procedure rtl_str_toAsciiUpperCase(str:Psal_Char); cdecl;

  {* Convert all ASCII lowercase letters to uppercase within a string.
  
      The characters with values between 97 and 122 (ASCII a--z) are replaced
      with values between 65 and 90 (ASCII A--Z).
  
      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.
  
      @param len
      the length of the string.
    }
  procedure rtl_str_toAsciiUpperCase_WithLength(str:Psal_Char; len:sal_Int32); cdecl;

  {* Remove white space from both ends of a string.

      All characters with values less than or equal to 32 (the space character)
      are considered to be white space.  This function cannot be used for
      language-specific operations.  The string must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @return
      the new length of the string.
    }
  function rtl_str_trim(str:Psal_Char):sal_Int32; cdecl;

  {* Remove white space from both ends of the string.
  
      All characters with values less than or equal to 32 (the space character)
      are considered to be white space.  This function cannot be used for
      language-specific operations.  The string must be null-terminated.
  
      @param str
      a string.  Need not be null-terminated, but must be at least as long as
      the specified len.

      @param len
      the original length of the string.
  
      @return
      the new length of the string.
    }
  function rtl_str_trim_WithLength(str:Psal_Char; len:sal_Int32):sal_Int32; cdecl;

  {* Create the string representation of a boolean.
  
      If b is true, the buffer is filled with the string "true" and 5 is
      returned.  If b is false, the buffer is filled with the string "false" and
      6 is returned.  This function cannot be used for language-specific
      operations.
  
      @param str
      a buffer that is big enough to hold the result and the terminating NUL
      character.  You should use the RTL_STR_MAX_VALUEOFBOOLEAN define to create
      a buffer that is big enough.
  
      @param b
      a boolean value.
  
      @return
      the length of the string.
    }
  function rtl_str_valueOfBoolean(str:Psal_Char; b:sal_Bool):sal_Int32; cdecl;


  const
     RTL_STR_MAX_VALUEOFBOOLEAN = 6;     
  {* Create the string representation of a character.
  
      @param str
      a buffer that is big enough to hold the result and the terminating NUL
      character.  You should use the RTL_STR_MAX_VALUEOFCHAR define to create a
      buffer that is big enough.
  
      @param ch
      a character value.
  
      @return
      the length of the string.
    }

  function rtl_str_valueOfChar(str:Psal_Char; ch:sal_Char):sal_Int32; cdecl;


  const
     RTL_STR_MAX_VALUEOFCHAR = 2;     
  {* Create the string representation of an integer.
  
      This function cannot be used for language-specific operations.
  
      @param str
      a buffer that is big enough to hold the result and the terminating NUL
      character.  You should use the RTL_STR_MAX_VALUEOFINT32 define to create a
      buffer that is big enough.
  
      @param i
      an integer value.
  
      @param radix
      the radix.  Must be between RTL_STR_MIN_RADIX (2) and RTL_STR_MAX_RADIX
      (36), inclusive.

      @return
      the length of the string.
    }

  function rtl_str_valueOfInt32(str:Psal_Char; i:sal_Int32; radix:sal_Int16):sal_Int32; cdecl;


  const
     RTL_STR_MIN_RADIX = 2;     
     RTL_STR_MAX_RADIX = 36;     
     RTL_STR_MAX_VALUEOFINT32 = 33;     
  {* Create the string representation of a long integer.
  
      This function cannot be used for language-specific operations.
  
      @param str
      a buffer that is big enough to hold the result and the terminating NUL
      character.  You should use the RTL_STR_MAX_VALUEOFINT64 define to create a
      buffer that is big enough.
  
      @param l
      a long integer value.
  
      @param radix
      the radix.  Must be between RTL_STR_MIN_RADIX (2) and RTL_STR_MAX_RADIX
      (36), inclusive.
  
      @return
      the length of the string.
    }

  function rtl_str_valueOfInt64(str:Psal_Char; l:sal_Int64; radix:sal_Int16):sal_Int32; cdecl;


  const
     RTL_STR_MAX_VALUEOFINT64 = 65;     
  {* Create the string representation of a float.
  
      This function cannot be used for language-specific conversion.
  
      @param str
      a buffer that is big enough to hold the result and the terminating NUL
      character.  You should use the RTL_STR_MAX_VALUEOFFLOAT define to create a
      buffer that is big enough.
  
      @param f
      a float value.
  
      @return
      the length of the string.
    }

  function rtl_str_valueOfFloat(str:Psal_Char; f:double):sal_Int32; cdecl;


  const
     RTL_STR_MAX_VALUEOFFLOAT = 15;     
  {* Create the string representation of a double.
  
      This function cannot be used for language-specific conversion.
  
      @param str
      a buffer that is big enough to hold the result and the terminating NUL
      character.  You should use the RTL_STR_MAX_VALUEOFDOUBLE define to create
      a buffer that is big enough.

      @param d
      a double value.
  
      @return
      the length of the string.
    }

  function rtl_str_valueOfDouble(str:Psal_Char; d:double):sal_Int32; cdecl;


  const
     RTL_STR_MAX_VALUEOFDOUBLE = 25;     
  {* Interpret a string as a boolean.
  
      This function cannot be used for language-specific conversion.  The string
      must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @return
      true if the string is "1" or "true" in any ASCII case, false otherwise.
    }
(* Const before type ignored *)

  function rtl_str_toBoolean(str:Psal_Char):sal_Bool; cdecl;

  {* Interpret a string as an integer.
  
      This function cannot be used for language-specific conversion.  The string
      must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @param radix
      the radix.  Must be between RTL_STR_MIN_RADIX (2) and RTL_STR_MAX_RADIX
      (36), inclusive.
  
      @return
      the integer value represented by the string, or 0 if the string does not
      represent an integer.
    }
(* Const before type ignored *)
  function rtl_str_toInt32(str:Psal_Char; radix:sal_Int16):sal_Int32; cdecl;

  {* Interpret a string as a long integer.
  
      This function cannot be used for language-specific conversion.  The string
      must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @param radix
      the radix.  Must be between RTL_STR_MIN_RADIX (2) and RTL_STR_MAX_RADIX
      (36), inclusive.
  
      @return
      the long integer value represented by the string, or 0 if the string does
      not represent a long integer.
    }
(* Const before type ignored *)
  function rtl_str_toInt64(str:Psal_Char; radix:sal_Int16):sal_Int64; cdecl;

  {* Interpret a string as a float.
  
      This function cannot be used for language-specific conversion.  The string
      must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @return
      the float value represented by the string, or 0.0 if the string does not
      represent a float.
    }
(* Const before type ignored *)
  function rtl_str_toFloat(str:Psal_Char):double; cdecl;

  {* Interpret a string as a double.
  
      This function cannot be used for language-specific conversion.  The string
      must be null-terminated.
  
      @param str
      a null-terminated string.
  
      @return
      the float value represented by the string, or 0.0 if the string does not
      represent a double.
    }
(* Const before type ignored *)
  function rtl_str_toDouble(str:Psal_Char):double; cdecl;

  { =======================================================================  }
  { -----------------------------------------------------------------------  }
  {* Increment the reference count of a string.
  
      @param str
      a string.
    }
  procedure rtl_string_acquire(str:Prtl_String); cdecl;

  {* Decrement the reference count of a string.
  
      If the count goes to zero than the string data is deleted.
  
      @param str
      a string.
    }
  procedure rtl_string_release(str:Prtl_String); cdecl;

  {* Allocate a new string containing no characters.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
    }
  procedure rtl_string_new(newStr: PPrtl_String); cdecl;

  {* Allocate a new string containing space for a given number of characters.
  
      If len is greater than zero, the reference count of the new string will be
      1.  The values of all characters are set to 0 and the length of the string
      is 0.  This function does not handle out-of-memory conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param len
      the number of characters.
    }
  procedure rtl_string_new_WithLength(newStr:PPrtl_String; len:sal_Int32); cdecl;

  {* Allocate a new string that contains a copy of another string.
  
      If the length of value is greater than zero, the reference count of the
      new string will be 1.  This function does not handle out-of-memory
      conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param value
      a valid string.
    }
(* Const before type ignored *)
  procedure rtl_string_newFromString(newStr:PPrtl_String; value:Prtl_String); cdecl;

  {* Allocate a new string that contains a copy of a character array.
  
      If the length of value is greater than zero, the reference count of the
      new string will be 1.  This function does not handle out-of-memory
      conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param value
      a null-terminated character array.
    }
(* Const before type ignored *)
  procedure rtl_string_newFromStr(newStr:PPrtl_String; value:Psal_Char);  cdecl;

  {* Allocate a new string that contains a copy of a character array.
  
      If the length of value is greater than zero, the reference count of the
      new string will be 1.  This function does not handle out-of-memory
      conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param value
      a character array.  Need not be null-terminated, but must be at least as
      long as the specified len.
  
      @param len
      the length of the character array.
    }
(* Const before type ignored *)
  procedure rtl_string_newFromStr_WithLength(newStr:PPrtl_String; value:Psal_Char; len:sal_Int32); cdecl;

  {* Assign a new value to a string.
  
      First releases any value str might currently hold, then acquires
      rightValue.
  
      @param str
      pointer to the string.  The pointed-to data must be null or a valid
      string.
  
      @param rightValue
      a valid string.
    }
  procedure rtl_string_assign(str:PPrtl_String; rightValue:Prtl_String); cdecl;

  {* Return the length of a string.
  
      The length is equal to the number of characters in the string.
  
      @param str
      a valid string.
  
      @return
      the length of the string.
    }
(* Const before type ignored *)
  function rtl_string_getLength(str:Prtl_String):sal_Int32; cdecl;

  {* Return a pointer to the underlying character array of a string.
  
      @param str
      a valid string.
  
      @return
      a pointer to the null-terminated character array.
    }
  function rtl_string_getStr(str:Prtl_String):Psal_Char; cdecl;

  {* Create a new string that is the concatenation of two other strings.
  
      The new string does not necessarily have a reference count of 1 (in cases
      where one of the two other strings is empty), so it must not be modified
      without checking the reference count.  This function does not handle
      out-of-memory conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param left
      a valid string.
  
      @param right
      a valid string.
    }
  procedure rtl_string_newConcat(newStr:PPrtl_String; left:Prtl_String; right:Prtl_String); cdecl;

  {* Create a new string by replacing a substring of another string.
  
      The new string results from replacing a number of characters (count),
      starting at the specified position (index) in the original string (str),
      with some new substring (subStr).  If subStr is null, than only a number
      of characters is deleted.
  
      The new string does not necessarily have a reference count of 1, so it
      must not be modified without checking the reference count.  This function
      does not handle out-of-memory conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param str
      a valid string.
  
      @param index
      the index into str at which to start replacement.  Must be between 0 and
      the length of str, inclusive.
  
      @param count
      the number of charcters to remove.  Must not be negative, and the sum of
      index and count must not exceed the length of str.
  
      @param subStr
      either null or a valid string to be inserted.
    }
  procedure rtl_string_newReplaceStrAt(newStr:PPrtl_String; str:Prtl_String; index:sal_Int32; count:sal_Int32; subStr:Prtl_String); cdecl;

  {* Create a new string by replacing all occurrences of a single character
      within another string.
  
      The new string results from replacing all occurrences of oldChar in str
      with newChar.
  
      The new string does not necessarily have a reference count of 1 (in cases
      where oldChar does not occur in str), so it must not be modified without
      checking the reference count.  This function does not handle out-of-memory
      conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param str
      a valid string.
  
      @param oldChar
      the old character.
  
      @param newChar
      the new character.
    }
  procedure rtl_string_newReplace(newStr:PPrtl_String; str:Prtl_String; oldChar:sal_Char; newChar:sal_Char); cdecl;

  {* Create a new string by converting all ASCII uppercase letters to lowercase
      within another string.
  
      The new string results from replacing all characters with values between
      65 and 90 (ASCII A--Z) by values between 97 and 122 (ASCII a--z).
  
      This function cannot be used for language-specific conversion.  The new
      string does not necessarily have a reference count of 1 (in cases where
      no characters need to be converted), so it must not be modified without
      checking the reference count.  This function does not handle out-of-memory
      conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param str
      a valid string.
    }
  procedure rtl_string_newToAsciiLowerCase(newStr:PPrtl_String; str:Prtl_String); cdecl;

  {* Create a new string by converting all ASCII lowercase letters to uppercase
      within another string.
  
      The new string results from replacing all characters with values between
      97 and 122 (ASCII a--z) by values between 65 and 90 (ASCII A--Z).
  
      This function cannot be used for language-specific conversion.  The new
      string does not necessarily have a reference count of 1 (in cases where
      no characters need to be converted), so it must not be modified without
      checking the reference count.  This function does not handle out-of-memory
      conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param str
      a valid string.
    }
  procedure rtl_string_newToAsciiUpperCase(newStr:PPrtl_String; str:Prtl_String); cdecl;

  {* Create a new string by removing white space from both ends of another
      string.
  
      The new string results from removing all characters with values less than
      or equal to 32 (the space character) form both ends of str.
  
      This function cannot be used for language-specific conversion.  The new
      string does not necessarily have a reference count of 1 (in cases where
      no characters need to be removed), so it must not be modified without
      checking the reference count.  This function does not handle out-of-memory
      conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.
  
      @param str
      a valid string.
    }
  procedure rtl_string_newTrim(newStr:PPrtl_String; str:Prtl_String); cdecl;

  {* Create a new string by extracting a single token from another string.
  
      Starting at index, the token's next token is searched for.  If there is no
      such token, the result is an empty string.  Otherwise, all characters from
      the start of that token and up to, but not including the next occurrence
      of cTok make up the resulting token.  The return value is the position of
      the next token, or -1 if no more tokens follow.
  
      Example code could look like
        rtl_String * pToken = NULL;
        sal_Int32 nIndex = 0;
        do
        
            ...
            nIndex = rtl_string_getToken(&pToken, pStr, 0, ';', nIndex);
            ...
        
        while (nIndex >= 0);
  
      The new string does not necessarily have a reference count of 1, so it
      must not be modified without checking the reference count.  This function
      does not handle out-of-memory conditions.
  
      @param newStr
      pointer to the new string.  The pointed-to data must be null or a valid
      string.  If either token or index is negative, an empty token is stored in
      newStr (and -1 is returned).
  
      @param str
      a valid string.
  
      @param token
      the number of the token to return, starting at index.
  
      @param cTok
      the character that seperates the tokens.
  
      @param index
      the position at which searching for the token starts.  Must not be greater
      than the length of str.
  
      @return
      the index of the next token, or -1 if no more tokens follow.
    }
  function rtl_string_getToken(newStr:PPrtl_String; str:Prtl_String; token:sal_Int32; cTok:sal_Char; index:sal_Int32):sal_Int32; cdecl;

  { =======================================================================  }
  {* Supply an ASCII string literal together with its length.
  
      This macro can be used to compute (some of) the arguments in function calls
      like rtl::OString(RTL_CONSTASCII_STRINGPARAM("foo")) or
      rtl::OUString::equalsAsciiL(RTL_CONSTASCII_STRINGPARAM("foo")).
  
      @param constAsciiStr
      must be an expression of type "(possibly cv-qualified reference to) array of
      (possibly cv-qualified) char."  Each element of the referenced array must
      represent an ASCII value in the range 0x00--0x7F.  The last element of the
      referenced array is not considered part of the represented ASCII string, and
      its value should be 0x00.  Depending on where this macro is used, the nature
      of the supplied expression might be further restricted.
   }
(* error 
#define RTL_CONSTASCII_STRINGPARAM( constAsciiStr ) constAsciiStr, ((sal_Int32)sizeof(constAsciiStr)-1)
in define line 1103 *)
    {* Supply the length of an ASCII string literal.
    
        This macro can be used to compute arguments in function calls like
        rtl::OUString::match(other, RTL_CONSTASCII_LENGTH("prefix")).
    
        @param constAsciiStr
        must be an expression of type "(possibly cv-qualified reference to) array of
        (possibly cv-qualified) char."  Each element of the referenced array must
        represent an ASCII value in the range 0x00--0x7F.  The last element of the
        referenced array is not considered part of the represented ASCII string, and
        its value should be 0x00.  Depending on where this macro is used, the nature
        of the supplied expression might be further restricted.
     }
    { was #define dname(params) para_def_expr }
    { argument types are unknown }
    function RTL_CONSTASCII_LENGTH(constAsciiStr : longint) : sal_Int32; cdecl;

    { =======================================================================  }
    { predefined constants for String-Conversion  }

{ 
    const
       OUSTRING_TO_OSTRING_CVTFLAGS = (((RTL_UNICODETOTEXT_FLAGS_UNDEFINED_DEFAULT or
                                         RTL_UNICODETOTEXT_FLAGS_INVALID_DEFAULT) or
                                         RTL_UNICODETOTEXT_FLAGS_UNDEFINED_REPLACE) or
                                         RTL_UNICODETOTEXT_FLAGS_PRIVATE_MAPTO0) or
                                         RTL_UNICODETOTEXT_FLAGS_NOCOMPOSITE;
}
    { -----------------------------------------------------------------------  }
    {* Create a new byte string by converting a Unicode string, using a specific
        text encoding.
    
        The lengths of the byte string and the Unicode string may differ (e.g.,
        for double-byte encodings, UTF-7, UTF-8).
    
        If the length of the Unicode string is greater than zero, the reference
        count of the new string will be 1.
    
        If an out-of-memory condition occurs, newStr will point to a null pointer
        upon return.
    
        @param newStr
        pointer to the new string.  The pointed-to data must be null or a valid
        string.
    
        @param str
        a Unicode character array.  Need not be null-terminated, but must be at
        least as long as the specified len.
    
        @param len
        the length of the Unicode character array.
    
        @param encoding
        the text encoding to use for conversion.
    
        @param convertFlags
        flags which control the conversion.  Either use
        OUSTRING_TO_OSTRING_CVTFLAGS, or see
        <http://udk.openoffice.org/cpp/man/spec/textconversion.html> for more
        details.
      }
(* Const before type ignored *)

    procedure rtl_uString2String(newStr:PPrtl_String; str:Psal_Unicode; len:sal_Int32; encoding: rtl_TextEncoding; convertFlags:sal_uInt32); cdecl;

    {*
      Converts a Unicode string to a byte string, signalling failure.
    
      @param pTarget
      An out parameter receiving the converted string.  Must not be null itself, and
      must contain either null or a pointer to a valid rtl_String; the contents are
      not modified if conversion fails (rtl_convertUStringToString returns false).
    
      @param pSource
      The Unicode string.  May only be null if nLength is zero.
    
      @param nLength
      The length of the Unicode string.  Must be non-negative.
    
      @param nEncoding
      The text encoding to convert into.  Must be an octet encoding (i.e.,
      rtl_isOctetTextEncoding(nEncoding) must return true).
    
      @param nFlags
      A combination of RTL_UNICODETOTEXT_FLAGS that detail how to do the conversion
      (see rtl_convertUnicodeToText).  RTL_UNICODETOTEXT_FLAGS_FLUSH need not be
      included, it is implicitly assumed.  Typical uses are either
      RTL_UNICODETOTEXT_FLAGS_UNDEFINED_ERROR |
      RTL_UNICODETOTEXT_FLAGS_INVALID_ERROR (fail if a Unicode character cannot be
      converted to the target nEncoding) or OUSTRING_TO_OSTRING_CVTFLAGS (make a
      best efforts conversion).
    
      @return
      True if the conversion succeeded, false otherwise.
      }
(* Const before declarator ignored *)
    function rtl_convertUStringToString(pTarget:PPrtl_String; pSource:Psal_Unicode; nLength:sal_Int32; nEncoding:rtl_TextEncoding; nFlags:sal_uInt32):sal_Bool; cdecl;

implementation

  function rtl_str_getLength(str:Psal_Char): sal_Int32; external sal3LibraryName;
  function rtl_str_compare(first:Psal_Char; second:Psal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_compare_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32):sal_Int32; external sal3LibraryName;
  function rtl_str_shortenedCompare_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32; shortenedLen:sal_Int32):sal_Int32; external sal3LibraryName;
  function rtl_str_reverseCompare_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32):sal_Int32; external sal3LibraryName;
  function rtl_str_compareIgnoreAsciiCase(first:Psal_Char; second:Psal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_compareIgnoreAsciiCase_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32):sal_Int32; external sal3LibraryName;
  function rtl_str_shortenedCompareIgnoreAsciiCase_WithLength(first:Psal_Char; firstLen:sal_Int32; second:Psal_Char; secondLen:sal_Int32; shortenedLen:sal_Int32):sal_Int32; external sal3LibraryName;
  function rtl_str_hashCode(str:Psal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_hashCode_WithLength(str:Psal_Char; len:sal_Int32):sal_Int32; external sal3LibraryName;
  function rtl_str_indexOfChar(str:Psal_Char; ch:sal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_indexOfChar_WithLength(str:Psal_Char; len:sal_Int32; ch:sal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_lastIndexOfChar(str:Psal_Char; ch:sal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_lastIndexOfChar_WithLength(str:Psal_Char; len:sal_Int32; ch:sal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_indexOfStr(str:Psal_Char; subStr:Psal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_indexOfStr_WithLength(str:Psal_Char; len:sal_Int32; subStr:Psal_Char; subLen:sal_Int32):sal_Int32; external sal3LibraryName;
  function rtl_str_lastIndexOfStr(str:Psal_Char; subStr:Psal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_lastIndexOfStr_WithLength(str:Psal_Char; len:sal_Int32; subStr:Psal_Char; subLen:sal_Int32):sal_Int32; external sal3LibraryName;
  procedure rtl_str_replaceChar(str:Psal_Char; oldChar:sal_Char; newChar:sal_Char); external sal3LibraryName;
  procedure rtl_str_replaceChar_WithLength(str:Psal_Char; len:sal_Int32; oldChar:sal_Char; newChar:sal_Char); external sal3LibraryName;
  procedure rtl_str_toAsciiLowerCase(str:Psal_Char); external sal3LibraryName;
  procedure rtl_str_toAsciiLowerCase_WithLength(str:Psal_Char; len:sal_Int32); external sal3LibraryName;
  procedure rtl_str_toAsciiUpperCase(str:Psal_Char); external sal3LibraryName;
  procedure rtl_str_toAsciiUpperCase_WithLength(str:Psal_Char; len:sal_Int32); external sal3LibraryName;
  function rtl_str_trim(str:Psal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_trim_WithLength(str:Psal_Char; len:sal_Int32):sal_Int32; external sal3LibraryName;
  function rtl_str_valueOfBoolean(str:Psal_Char; b:sal_Bool):sal_Int32; external sal3LibraryName;
  function rtl_str_valueOfChar(str:Psal_Char; ch:sal_Char):sal_Int32; external sal3LibraryName;
  function rtl_str_valueOfInt32(str:Psal_Char; i:sal_Int32; radix:sal_Int16):sal_Int32; external sal3LibraryName;
  function rtl_str_valueOfInt64(str:Psal_Char; l:sal_Int64; radix:sal_Int16):sal_Int32; external sal3LibraryName;
  function rtl_str_valueOfFloat(str:Psal_Char; f:double):sal_Int32; external sal3LibraryName;
  function rtl_str_valueOfDouble(str:Psal_Char; d:double):sal_Int32; external sal3LibraryName;
  function rtl_str_toBoolean(str:Psal_Char):sal_Bool; external sal3LibraryName;
  function rtl_str_toInt32(str:Psal_Char; radix:sal_Int16):sal_Int32; external sal3LibraryName;
  function rtl_str_toInt64(str:Psal_Char; radix:sal_Int16):sal_Int64; external sal3LibraryName;
  function rtl_str_toFloat(str:Psal_Char):double; external sal3LibraryName;
  function rtl_str_toDouble(str:Psal_Char):double; external sal3LibraryName;
  procedure rtl_string_acquire(str:Prtl_String); external sal3LibraryName;
  procedure rtl_string_release(str:Prtl_String); external sal3LibraryName;
  procedure rtl_string_new(newStr:PPrtl_String); external sal3LibraryName;
  procedure rtl_string_new_WithLength(newStr:PPrtl_String; len:sal_Int32); external sal3LibraryName;
  procedure rtl_string_newFromString(newStr:PPrtl_String; value:Prtl_String); external sal3LibraryName;
  procedure rtl_string_newFromStr(newStr:PPrtl_String; value:Psal_Char); external sal3LibraryName;
  procedure rtl_string_newFromStr_WithLength(newStr:PPrtl_String; value:Psal_Char; len:sal_Int32); external sal3LibraryName;
  procedure rtl_string_assign(str:PPrtl_String; rightValue:Prtl_String); external sal3LibraryName;
  function rtl_string_getLength(str:Prtl_String):sal_Int32; external sal3LibraryName;
  function rtl_string_getStr(str:Prtl_String):Psal_Char; external sal3LibraryName;
  procedure rtl_string_newConcat(newStr:PPrtl_String; left:Prtl_String; right:Prtl_String); external sal3LibraryName;
  procedure rtl_string_newReplaceStrAt(newStr:PPrtl_String; str:Prtl_String; index:sal_Int32; count:sal_Int32; subStr:Prtl_String); external sal3LibraryName;
  procedure rtl_string_newReplace(newStr:PPrtl_String; str:Prtl_String; oldChar:sal_Char; newChar:sal_Char); external sal3LibraryName;
  procedure rtl_string_newToAsciiLowerCase(newStr:PPrtl_String; str:Prtl_String); external sal3LibraryName;
  procedure rtl_string_newToAsciiUpperCase(newStr:PPrtl_String; str:Prtl_String); external sal3LibraryName;
  procedure rtl_string_newTrim(newStr:PPrtl_String; str:Prtl_String); external sal3LibraryName;
  function rtl_string_getToken(newStr:PPrtl_String; str:Prtl_String; token:sal_Int32; cTok:sal_Char; index:sal_Int32):sal_Int32; external sal3LibraryName;
  procedure rtl_uString2String(newStr:PPrtl_String; str:Psal_Unicode; len:sal_Int32; encoding:rtl_TextEncoding; convertFlags:sal_uInt32); external sal3LibraryName;
  function rtl_convertUStringToString(pTarget:PPrtl_String; pSource:Psal_Unicode; nLength:sal_Int32; nEncoding:rtl_TextEncoding; nFlags:sal_uInt32):sal_Bool; external sal3LibraryName;
  { was #define dname(params) para_def_expr }
  { argument types are unknown }
  function RTL_CONSTASCII_LENGTH(constAsciiStr : longint) : sal_Int32;
    begin
       RTL_CONSTASCII_LENGTH := sal_Int32((sizeof(constAsciiStr)) - 1);
    end;

end.
