unit uno_sal_types;

interface

{$I uno_common.inc}

uses
  uno_sal_config;

const
  SAL_TYPES_ALIGNMENT2     = 1;
  SAL_TYPES_ALIGNMENT4     = 1;
  SAL_TYPES_ALIGNMENT8     = 1;
  SAL_TYPES_SIZEOFSHORT    = 2;
  SAL_TYPES_SIZEOFINT      = 4;
  SAL_TYPES_SIZEOFLONG     = 4;
  SAL_TYPES_SIZEOFLONGLONG = 8;
  SAL_TYPES_SIZEOFPOINTER  = 4;

type
  (********************************************************************************)
  (************************************ Data types ********************************)
  (********************************************************************************)
  // Boolean
//  sal_Bool = char;
  sal_Bool = ByteBool;
  psal_Bool =^sal_Bool;

const
  sal_False: sal_Bool = sal_Bool(0);
  sal_True: sal_Bool = sal_Bool(1);

type
// 
  PPPointer = ^PPointer;

//  sal_Int8 = char;
  sal_Int8 = SmallInt;
  psal_Int8 = ^sal_Int8;

//  sal_uInt8 = char;
  sal_uInt8 = Byte;
  psal_uInt8 = ^sal_uInt8;

  {$if SAL_TYPES_SIZEOFSHORT = 2}
    sal_Int16 = smallint;
    sal_uInt16 = word;
  {$else}
    {$error "Could not find 16-bit type, add support for your architecture"}
  {$ifend}
  Psal_Int16 = ^sal_Int16;
  Psal_uInt16 = ^sal_uInt16;

  {$if SAL_TYPES_SIZEOFLONG = 4}
    sal_Int32 = LongInt;
    sal_uInt32 = LongWord;
  {$elseif SAL_TYPES_SIZEOFINT = 4}
    sal_Int32 = integer;
    sal_uInt32 = UINT;
  {$else}
    {$error "Could not find 32-bit type, add support for your architecture"}
  {$ifend}
  Psal_Int32 = ^sal_Int32;
  Psal_uInt32 = ^sal_uInt32;

//  The following are macros that will add the 64 bit constant suffix.
//  function SAL_CONST_INT64(x: LongInt): LongInt;
//  function SAL_CONST_UINT64(x: LongWord): LongWord;
type
  {$if SAL_TYPES_SIZEOFLONGLONG = 8}
    sal_Int64 = Int64;
    sal_uInt64 = Int64;
  {$elseif SAL_TYPES_SIZEOFLONG = 8}
    sal_Int64 = LonigInt;
    sal_uInt64 = LongWord;
  {$else}
    {$error "Could not find 64-bit type, add support for your architecture"}
  {$ifend}
  Psal_Int64 = ^sal_Int64;
  Psal_uInt64 = ^sal_uInt64;

  sal_Char = AnsiChar;
  psal_Char = ^sal_Char;

  sal_sChar = AnsiChar;
  psal_sChar = ^sal_sChar;

  sal_uChar = Char;
  psal_uChar = ^sal_uChar;

  sal_Unicode = WideChar;
  psal_Unicode = ^sal_Unicode;

  sal_Handle = pointer;
  psal_Handle = ^sal_Handle;

  // sal_Size should currently be the native width of the platform
  {$if SAL_TYPES_SIZEOFPOINTER = 4}
    sal_Size = sal_uInt32;
    sal_sSize = sal_Int32;
  {$elseif SAL_TYPES_SIZEOFPOINTER = 8}
    sal_Size = sal_uInt64;
    sal_sSize = sal_Int64;
  {$else}
    {$error "Please make sure SAL_TYPES_SIZEOFPOINTER is defined for your architecture/compiler"}
  {$ifend}
  psal_Size  = ^sal_Size;

  // sal_PtrDiff holds the result of a pointer subtraction
  {$if SAL_TYPES_SIZEOFPOINTER = 4}
    sal_PtrDiff = sal_Int32;
  {$elseif SAL_TYPES_SIZEOFPOINTER = 8}
    sal_PtrDiff = sal_Int64;
  {$else}
    {$error "Please make sure SAL_TYPES_SIZEOFPOINTER is defined for your architecture/compiler"}
  {$ifend}
  // sal_IntPtr, sal_uIntPtr are integer types designed to hold pointers so that any valid
  // pointer to void can be converted to this type and back to a pointer to void and the
  // result will compare to the original pointer
  {$if SAL_TYPES_SIZEOFPOINTER = 4}
    sal_IntPtr = sal_Int32;
    sal_uIntPtr = sal_uInt32;
  {$elseif SAL_TYPES_SIZEOFPOINTER = 8}
    sal_IntPtr = sal_Int64;
    sal_uIntPtr = sal_uInt64;
  {$else}
    {$error "Please make sure SAL_TYPES_SIZEOFPOINTER is defined for your architecture/compiler"}
  {$ifend}

  //****************************************************************************
  // Useful defines *)
  // The following SAL_MIN_INTn defines codify the assumption that the signed
  // sal_Int types use two's complement representation.  Defining them as
  // "-0x7F... - 1" instead of as "-0x80..." prevents warnings about applying the
  // unary minus operator to unsigned quantities.
  //****************************************************************************
const
  SAL_MIN_INT8 = sal_Int8(-$7F-1);
  SAL_MAX_INT8 = sal_Int8($7F);
  SAL_MAX_UINT8 = sal_uInt8($FF);
  SAL_MIN_INT16 = sal_Int16(-$7FFF-1);
  SAL_MAX_INT16 = sal_Int16($7FFF);
  SAL_MAX_UINT16 = sal_uInt16($FFFF);
  SAL_MIN_INT32 = sal_Int32(-$7FFFFFFF-1);
  SAL_MAX_INT32 = sal_Int32($7FFFFFFF);
  SAL_MAX_UINT32 = sal_uInt32($FFFFFFFF);
  SAL_MIN_INT64 = sal_Int64(Low(Int64));
  SAL_MAX_INT64 = sal_Int64(High(Int64));
  SAL_MAX_UINT64 = sal_uInt64(High(Int64));

  {$if defined(SAL_W32) or defined(SAL_OS2) or defined(SAL_UNX)}
    SAL_MAX_ENUM = High(LongInt);
  {$elseif defined(SAL_W16)}
    SAL_MAX_ENUM = $7fff;
  {$ifend}
(*
  {$ifdef _MSC_VER}
    SAL_DLLPUBLIC_EXPORT = __declspec(dllexport);
    SAL_DLLPUBLIC_IMPORT = __declspec(dllimport);
    {$define SAL_DLLPRIVATE}
    SAL_CALL = __cdecl;
    SAL_CALL_ELLIPSE = __cdecl;
  {$elseif defined(SAL_OS2)}
    {$define SAL_DLLPUBLIC_EXPORT}
    {$define SAL_DLLPUBLIC_IMPORT}
    {$define SAL_DLLPRIVATE}
    {$define SAL_CALL}
    {$define SAL_CALL_ELLIPSE}
  {$elseif defined(SAL_UNX)}
    {$if defined(__SUNPRO_CC) && (__SUNPRO_CC >= 0x550)}
       SAL_DLLPUBLIC_EXPORT = __global;
       {$define SAL_DLLPUBLIC_IMPORT}
       SAL_DLLPRIVATE = __hidden;
    {$elif defined(__SUNPRO_C ) && (__SUNPRO_C  >= 0x550)}
       SAL_DLLPUBLIC_EXPORT = __global;
       {$define SAL_DLLPUBLIC_IMPORT}
       SAL_DLLPRIVATE = __hidden;
    {$elif defined(__GNUC__) && defined(HAVE_GCC_VISIBILITY_FEATURE)}
       SAL_DLLPUBLIC_EXPORT = __attribute__((visibility('default')));
       {$define SAL_DLLPUBLIC_IMPORT}
       SAL_DLLPRIVATE = __attribute__((visibility('hidden')));
    {$else}
      {$define SAL_DLLPUBLIC_EXPORT}
      {$define SAL_DLLPUBLIC_IMPORT}
      {$define SAL_DLLPRIVATE}
    {$ifend}
    {$define SAL_CALL}
    {$define SAL_CALL_ELLIPSE}
  {$else}
    {$error "unknown platform"}
  {$endif}
*)
  // Use this for pure virtual classes, e.g. class SAL_NO_VTABLE Foo { ...
  // This hinders the compiler from setting a generic vtable stating that
  // a pure virtual function was called and thus slightly reduces code size.
  {$ifdef _MSC_VER}
    SAL_NO_VTABLE = __declspec(novtable);
  {$else}
    {$define SAL_NO_VTABLE}
  {$endif}

{$A8}

type
  _sal_Sequence = record
    nRefCount: sal_Int32;
    nElements: sal_Int32;
//    elements: array[0..0] of sal_Char;
    elements: Pointer;
  end;
  sal_Sequence = _sal_Sequence;
  psal_Sequence = ^sal_Sequence;
  ppsal_Sequence = ^psal_Sequence;


const

//  SAL_SEQUENCE_HEADER_SIZE = sal_Size(@psal_Sequence^.elements);
// #define SAL_SEQUENCE_HEADER_SIZE ((sal_Size)&((sal_Sequence *)0)->elements)

  SAL_SEQUENCE_HEADER_SIZE = 0;
//    Wrap C++ const_cast, reinterpret_cast and static_cast expressions in
//    macros to keep code portable to old compilers (since most compilers still
//    lack RTTI support, dynamic_cast is not included here).
{$ifdef __cplusplus}
  {$if defined(SAL_W32) or defined(SOLARIS) or defined(LINUX) or defined(MACOSX) or defined(FREEBSD) or defined(NETBSD) or defined(AIX)}
//    function SAL_CONST_CAST(type: integer; expr: integer): integer;
//    function SAL_REINTERPRET_CAST(type: integer; expr: integer): integer;
//    function SAL_STATIC_CAST(type: integer; expr: integer): integer;
  {$else}
    // SAL_W32, SOLARIS, LINUX
//    function SAL_CONST_CAST(type: integer; expr: integer): integer;
//    function SAL_REINTERPRET_CAST(type: integer; expr: integer): integer;
//    function SAL_STATIC_CAST(type: integer; expr: integer): integer;
  {$ifend}
{$endif}

(* __cplusplus *)
(** Definition of function throw clause macros.  These have been introduced
 to reduce code size by balancing out compiler bugs.

 These macros are ONLY for function declarations,
 use common C++ throw statement for throwing exceptions, e.g.
 throw RuntimeException();

 SAL_THROW()    should be used for all C++ functions, e.g. SAL_THROW( () )
 SAL_THROW_EXTERN_C() should be used for all C functions
*)
(*
{$ifdef __cplusplus}
  {$if defined(__GNUC__) || defined(__SUNPRO_CC) || defined(__sgi)}
    function SAL_THROW(exc: integer): integer; {<= !!!6 unknown macro}
  {$else}
    function SAL_THROW(exc: integer): integer; {<= !!!6 unknown macro}
  {$endif}
  // __GNUC__, __SUNPRO_CC
  function SAL_THROW_EXTERN_C(): integer;
{$else}
  // SAL_THROW() must not be used in C headers, only SAL_THROW_EXTERN_C() is defined
  function SAL_THROW_EXTERN_C(): integer; {<= !!!6 unknown macro}
{$endif}

{$ifdef __cplusplus}
enum
begin
  // definition of a no acquire enum for ctors
  SAL_NO_ACQUIRE{!!!e unknown token}
end;
{$endif}
// __cplusplus
{$ifdef __cplusplus}
;
end;
{$endif}
*)
implementation
(*
{SAL_CONST_INT64(x)       x##i64}
function SAL_CONST_INT64(x: integer): Integer;
begin
  result:=IntToStr(x)+'i64';
end;

{SAL_CONST_UINT64(x)      x##ui64}
function SAL_CONST_UINT64(x: integer): String;
begin
  result:=IntToStr(x)+'ui64';
end;

{SAL_CONST_INT64(x)       x##ll}
function SAL_CONST_INT64(x: integer): String;
begin
  result:=IntToStr(x)+'ll';
end;

{SAL_CONST_UINT64(x)      x##ull}
function SAL_CONST_UINT64(x: integer): String;
begin
  result:=IntToStr(x)+'ull';
end;

{SAL_CONST_INT64(x)       x##l}
function SAL_CONST_INT64(x: LongInt): LongInt;
begin
  result:=x;
end;

{SAL_CONST_UINT64(x)      x##ul}
function SAL_CONST_UINT64(x: LongWord): LongWord;
begin
  result:=x;
end;

{SAL_CONST_CAST(type, expr) (const_cast< type >(expr))}
function SAL_CONST_CAST(type: integer; expr: integer): integer;
begin
  result:=(const_cast<type>(expr));
end;

{SAL_REINTERPRET_CAST(type, expr) (reinterpret_cast< type >(expr))}
function SAL_REINTERPRET_CAST(type: integer; expr: integer): integer;
begin
  result:=(reinterpret_cast<type>(expr));
end;

{SAL_STATIC_CAST(type, expr) (static_cast< type >(expr))}
function SAL_STATIC_CAST(type: integer; expr: integer): integer;
begin
  result:=(static_cast<type>(expr))
end;

{SAL_CONST_CAST(type, expr) ((type) (expr))}
function SAL_CONST_CAST(type: integer; expr: integer): integer;
begin
  result:=({!!!a type cast? =>} {type(}(expr))
end;

{SAL_REINTERPRET_CAST(type, expr) ((type) (expr))}
function SAL_REINTERPRET_CAST(type: integer; expr: integer): integer;
begin
  result:=({!!!a type cast? =>} {type(}(expr))
end;

{SAL_STATIC_CAST(type, expr) ((type) (expr))}
function SAL_STATIC_CAST(type: integer; expr: integer): integer;
begin
  result:=({!!!a type cast? =>} {type(}(expr))
end;

{SAL_THROW( exc )}
function SAL_THROW(exc: integer): integer;
begin
end;

{SAL_THROW( exc ) throw exc}
function SAL_THROW(exc: integer): integer;
begin
  result:=throwexc
end;

{SAL_THROW_EXTERN_C() throw ()}
function SAL_THROW_EXTERN_C(): integer;
begin
  result:=throw()
end;

{SAL_THROW_EXTERN_C()}
function SAL_THROW_EXTERN_C(): integer;
begin
end;
*)

end.

