unit uno_sequence2;

interface

{$I uno_common.inc}

uses
   uno_sal_config
  ,uno_sal_types
  ,uno_data
  ,uno_typelib_typedescription;

type
    uno_Sequence = sal_Sequence;
    puno_Sequence = ^uno_Sequence;
    ppuno_Sequence = ^puno_Sequence;

    { Assigns a sequence.
     @param ppDest   destinstaion sequence
     @param pSource  source sequence
     @param pTypeDescr type description of the sequence and NOT of an element
     @param release  function called each time an interface needs to
                            be released; defaults (0) to uno       }
    procedure uno_sequence_assign( var pDest: puno_Sequence; pSource: puno_Sequence; pTypeDescr: p_typelib_TypeDescription;
                                    release: uno_ReleaseFunc ); cdecl;

    { Assigns a sequence.

     @param ppDest  destinstaion sequence
     @param pSource  source sequence
     @param pType  type of the sequence and NOT of an element
     @param release  function called each time an interface needs to
                            be released; defaults (0) to uno       }
    procedure uno_type_sequence_assign( var pDest: puno_Sequence; pSource: puno_Sequence; pTypeDescr: p_typelib_TypeDescription;
                                         release: uno_ReleaseFunc ); cdecl;

    { Constructs a new sequence with given elements.
     @param ppSequence    out parameter sequence; 0 if memory allocation has failed
     @param pTypeDescr    type description of the sequence and NOT of an element
     @param pElements     if 0, then all elements are default constructed
     @param len           number of elements
     @param acquire       function called each time an interface needs to be acquired; defaults (0) to uno
        @return           false, if memoray allocation has failed     }
    function uno_sequence_construct( var pSequence: puno_Sequence; pTypeDescr: p_typelib_TypeDescription; pElements: Pointer;
                                      len: sal_Int32; acquire: uno_AcquireFunc ): sal_Bool; cdecl;

    { Constructs a new sequence with given elements.
     @param ppSequence    out parameter sequence; 0 if memory allocation has failed
     @param pType         type of the sequence and NOT of an element
     @param pElements     if 0, then all elements are default constructed
     @param len           number of elements
     @param acquire       function called each time an interface needs to be acquired; defaults (0) to uno
        @return                   false, if memoray allocation has failed
    }
    function uno_type_sequence_construct( var pSequence: puno_Sequence; pType: p_typelib_TypeDescriptionReference; pElements: Pointer;
                                      len: sal_Int32; acquire: uno_AcquireFunc ): sal_Bool; cdecl;

    { Assures that the reference count of the given sequence is one.
        Otherwise a new copy of the sequence is created with a reference count
        of one.

     @param ppSequence  inout sequence
     @param pTypeDescr  type description of sequence
     @param acquire   function called each time an interface needs to
                                be acquired; defaults (0) to uno
     @param release   function called each time an interface needs to
                                be released; defaults (0) to uno
        @return                 false, if memoray allocation has failed
    }
    function uno_sequence_reference2One( var pDest: puno_Sequence; pTypeDescr: p_typelib_TypeDescription;
                                          acquire: uno_AcquireFunc; release: uno_ReleaseFunc ): sal_Bool; cdecl;

    { Assures that the reference count of the given sequence is one.
        Otherwise a new copy of the sequence is created with a reference count
        of one.

     @param ppSequence  inout sequence
     @param pType   type of sequence
     @param acquire   function called each time an interface needs to
                                be acquired; defaults (0) to uno
     @param release   function called each time an interface needs to
                                be released; defaults (0) to uno
        @return                 false, if memoray allocation has failed
    }
    function uno_type_sequence_reference2One( var pDest: puno_Sequence; pTypeDescr: p_typelib_TypeDescription;
                                               acquire: uno_AcquireFunc; release: uno_ReleaseFunc ): sal_Bool; cdecl;

    { Reallocates length of a sequence. This truncates a sequence or enlarges
        it default constructing appended elements.

     @param ppSequence  inout sequence
     @param pTypeDescr  type description of sequence
     @param nSize   new size of sequence
     @param acquire   function called each time an interface needs to
                                be acquired; defaults (0) to uno
     @param release   function called each time an interface needs to
                                be released; defaults (0) to uno
        @return                 false, if memoray allocation has failed
    }
    function uno_sequence_realloc( var pDest: puno_Sequence; pTypeDescr: p_typelib_TypeDescription; nSize: sal_Int32;
                                    acquire: uno_AcquireFunc; release: uno_ReleaseFunc ): sal_Bool; cdecl;

    { Reallocates length of a sequence. This truncates a sequence or enlarges
        it default constructing appended elements.

     @param ppSequence  inout sequence
     @param pType   type of sequence
     @param nSize   new size of sequence
     @param acquire   function called each time an interface needs to
                                be acquired; defaults (0) to uno
     @param release   function called each time an interface needs to
                                be released; defaults (0) to uno
        @return                 false, if memoray allocation has failed
    }
    function uno_type_sequence_realloc( var pDest: puno_Sequence; pTypeDescr: p_typelib_TypeDescription; nSize: sal_Int32;
                                         acquire: uno_AcquireFunc; release: uno_ReleaseFunc ): sal_Bool; cdecl;

implementation

    procedure uno_sequence_assign( var pDest: puno_Sequence; pSource: puno_Sequence; pTypeDescr: p_typelib_TypeDescription;
                                   release: uno_ReleaseFunc ); external cppu3LibraryName;
    procedure uno_type_sequence_assign( var pDest: puno_Sequence; pSource: puno_Sequence; pTypeDescr: p_typelib_TypeDescription;
                                        release: uno_ReleaseFunc ); external cppu3LibraryName;
    function uno_sequence_construct( var pSequence: puno_Sequence; pTypeDescr: p_typelib_TypeDescription; pElements: Pointer;
                                     len: sal_Int32; acquire: uno_AcquireFunc ): sal_Bool; external cppu3LibraryName;
    function uno_type_sequence_construct( var pSequence: puno_Sequence; pType: p_typelib_TypeDescriptionReference; pElements: Pointer;
                                      len: sal_Int32; acquire: uno_AcquireFunc ): sal_Bool; external cppu3LibraryName;
    function uno_sequence_reference2One( var pDest: puno_Sequence; pTypeDescr: p_typelib_TypeDescription;
                                          acquire: uno_AcquireFunc; release: uno_ReleaseFunc ): sal_Bool; external cppu3LibraryName;
    function uno_type_sequence_reference2One( var pDest: puno_Sequence; pTypeDescr: p_typelib_TypeDescription;
                                               acquire: uno_AcquireFunc; release: uno_ReleaseFunc ): sal_Bool; external cppu3LibraryName;
    function uno_sequence_realloc( var pDest: puno_Sequence; pTypeDescr: p_typelib_TypeDescription; nSize: sal_Int32;
                                    acquire: uno_AcquireFunc; release: uno_ReleaseFunc ): sal_Bool; external cppu3LibraryName;
    function uno_type_sequence_realloc( var pDest: puno_Sequence; pTypeDescr: p_typelib_TypeDescription; nSize: sal_Int32;
                                         acquire: uno_AcquireFunc; release: uno_ReleaseFunc ): sal_Bool; external cppu3LibraryName;  

end.
