unit uno_shared;

interface

{$I uno_common.inc}

uses
  Classes
  ,Math
  ,Types
  ,uno_rtl_ustring
  ,uno_sal_types
  ,uno_typelib_typedescription
  ,uno_typelib_typeclass
  ;
type

//==============================================================================
  TObjectsList = class
  protected
    fObjects: array of TObject;
    fOrdered: Boolean;
    fOnChanged: TNotifyEvent;

    procedure AfterConstruction; override;

    function GetObject(Index: Integer): TObject;
    function GetCount: Integer;
    procedure SetOrdered(Value: Boolean);
    procedure DoChanged;
  public
    procedure CheckIndex(Index: Integer);
    function Add(Obj: TObject): Integer; virtual;
    procedure Insert(AfterThis: Integer; Obj: TObject); virtual;
    procedure Clear; virtual;
    procedure FreeAndClear; virtual;
    procedure Assign(Src: TObjectsList); virtual;
    procedure Exchange(First, Second: Integer); virtual;
    procedure Remove(Index: Integer); overload; virtual;
    procedure Remove(Obj: TObject); overload; virtual;
    function Find(Obj: TObject): Integer;
    function GetIntersection(WithThis, Output: TObjectsList): Integer;

    property Objects[Index: Integer]: TObject read GetObject; default;
    property Count: Integer read GetCount;
    property Ordered: Boolean read fOrdered write SetOrdered;
    property OnChanged: TNotifyEvent read fOnChanged write fOnChanged;
  end;
//------------------------------------------------------------------------------

//==============================================================================

// working with Type Description
  procedure GetTypeDescriptionByName(var pTypeDesc: p_typelib_TypeDescription; AName: AnsiString);
  function GetParamSizeByVarType( AvarType: Word ): Integer;
  function GetMemberTypeDescByName( pInterfaceTypeDescription: p_typelib_InterfaceTypeDescription;
                                    aMemberName: AnsiString ): p_typelib_InterfaceMemberTypeDescription;
  procedure FreeMemberTypeDesc(pInterfaceMemberTypeDescription: p_typelib_InterfaceMemberTypeDescription);

// working with strings
  function String_to_rtl_uString(AStr: WideString): prtl_uString;
  function rtl_uString_to_String(pArtl_uStr: prtl_uString): WideString;
  function compare_rtl_uString_AnsiString(prtlStr: prtl_uString; Str: AnsiString): boolean;

//
  function CheckReducibility(AUNOType: typelib_TypeClass; AvarType: TVarType): boolean;
  function GetUNOTypeClassByPascalType(const aType: Integer; const aPTR: Pointer): typelib_TypeClass;

  function GetUNOTypeDescriptionReferenceByPascalType(const aPascalType: Integer;
                                                      const aPointer: Pointer): p_typelib_TypeDescriptionReference;
  function GetUNOTypeDescriptionByPascalType( const aPascalType: Integer;
                                              const aPointer: Pointer): p_typelib_TypeDescription;

  function NeedPassByRef(pParamTD: p_typelib_TypeDescriptionReference): Boolean;
  function AlignSize(pParamTD: p_typelib_TypeDescription): Integer;
//------------------------------------------------------------------------------
  function NeedNoCopy( pParamTD: p_typelib_TypeDescriptionReference ): Boolean;
  function HasWrapper( ATypeClass: typelib_TypeClass): Boolean;

  function DestroyData_and_FreeMem(var ADataPtr: Pointer; ADT: p_typelib_TypeDescriptionReference): Boolean;
  function EndTypeClass(pTD: p_typelib_TypeDescription; pData: Pointer): typelib_TypeClass;

  procedure PascalParam2UNOParam( pPascalParam: Pointer;
                                  APascalParamType: Integer;
                                  pUNOParamTD: p_typelib_TypeDescriptionReference;
                                  var pOutUNOParam: Pointer;
                                  var AUNOSize: Integer;
                                  NeverMove: Boolean = False);
  procedure UnoParam2PascalParam( AUnoParamPointer: Pointer;
                                  AUnoParamTypeDesc: p_typelib_TypeDescriptionReference;
                                  var APascalPointer: Pointer;
                                  var APascalParam: Variant;
                                  Reserved: Pointer = nil);
//------------------------------------------------------------------------------
const

  sQueryInterface: string = 'queryinterface';

  sError_PascalEnvFailed = 'Pascal enviroment not loaded';
  sError_UnoEnvFailed = 'UNO enviroment not loaded';
  sError_CantGetUnoMapping = 'Can not get UNO mapping';
  sError_CantGetPascalMapping = 'Can not get Pascal mapping';
  
  sError_PascalProxy_DispInvoke_NonExistentMember = 'The interface "%s" has no member "%s"';
  sError_PascalProxy_DispInvoke_NotEqualParamCount = 'Not equal param count';

  sError_MethodHasNoReturn = 'UNO method has no return';
  sError_ErrorInParams = 'Error in params';
  sError_ErrorInParamType = 'Type of parameter is false';

  sError_Bootstrap = 'Error in bootstrap. Can not find XComponentContext proxy interface';
  sError_PascalProxy_CantCreate_ErrorInInterfaceTypeDescription = 'Can not create Pascal Proxy. Error in InterfaceTypeDescription';
  sError_PascalProxy_CantCreateWithoutBridge = 'Can not create Pascal Proxy without bridge';

  sError_OOStructure_CantGetTypeDescription = 'Can not get TypeDescription of structure %s';
  sError_OOType_CantGetTypeDescription = 'Can not get TypeDescription of type %s';

  sError_UNOQueryInterface_CantGetTypeDescriptionWithName = 'Can not get TypeDescription with name %s.';
  sError_UNOQueryInterface_CantCompleteTypeDescriptionWithName = 'Can not complete TypeDescription with name %s.';

//------------------------------------------------------------------------------

implementation

uses
  //
  Variants
  ,SysUtils
  //
  ,uno_sequence2
  ,uno_any2
  ,uno_data
  ,uno_pascal_uno
  ,uno_pascal_proxy
  ,uno_pascal_struct
  ,uno_pascal_type
  ;

//------------------------------------------------------------------------------
//============================== TObjectsList ==================================
//------------------------------------------------------------------------------
procedure TObjectsList.AfterConstruction;
begin
  fOrdered := False;
  inherited AfterConstruction;
end;

procedure TObjectsList.CheckIndex(Index: Integer);
begin
  Assert((Index>0) and (Index<=Count), ClassName+': bad object index: '+IntToStr(Index));
end;

function TObjectsList.GetObject(Index: Integer): TObject;
begin
  CheckIndex(Index);
  Result := fObjects[Index-1];
end;

function TObjectsList.GetCount: Integer;
begin
  Result := Length(fObjects);
end;

procedure TObjectsList.SetOrdered(Value: Boolean);

  procedure QuickSort(iLo, iHi: Longword);
  var
    Lo, Hi: Longword;
    MidObj, TmpObj: TObject;
  begin
    Lo := iLo;
    Hi := iHi;
    MidObj := fObjects[(Lo+Hi) div 2];
    repeat
      while Longword(fObjects[Lo])<Longword(MidObj) do Inc(Lo);
      while Longword(fObjects[Hi])>Longword(MidObj) do Dec(Hi);
      if Lo<=Hi then begin
        TmpObj := fObjects[Lo];
        fObjects[Lo] := fObjects[Hi];
        fObjects[Hi] := TmpObj;
        Inc(Lo);
        Dec(Hi);
      end;
    until Lo>Hi;
    if Hi>iLo then QuickSort(iLo, Hi);
    if Lo<iHi then QuickSort(Lo, iHi);
  end;

begin
  if (not fOrdered) and Value and (Count>1) then begin
    QuickSort(0, Count-1);
    DoChanged;
  end;
  fOrdered := Value;
end;

procedure TObjectsList.DoChanged;
begin
  if Assigned(fOnChanged) then fOnChanged(Self);
end;

function TObjectsList.Add(Obj: TObject): Integer;
var
  Lo, Hi, Mid: Longword;
begin
  if fOrdered and (Count>0) then begin
    Lo := 0;
    Hi := Count-1;
    repeat
      Mid := (Hi+Lo) div 2;
      if Longword(Obj)>Longword(fObjects[Mid])
        then Lo := Mid
        else Hi := Mid;
    until (Hi-Lo)<=1;
    if Longword(Obj)<Longword(fObjects[Lo])
      then Result := Lo+1
    else if Longword(Obj)<Longword(fObjects[Hi])
      then Result := Hi+1
      else Result := Hi+2;

    Lo := Count+1;
    SetLength(fObjects, Lo);
    if Result<Lo
      then Move(fObjects[Result-1], fObjects[Result], (Lo-Result)*SizeOf(TObject));
  end else begin
    Result := Count+1;
    SetLength(fObjects, Result);
  end;
  fObjects[Result-1] := Obj;
  DoChanged;
end;

procedure TObjectsList.Insert(AfterThis: Integer; Obj: TObject);
var
  i: Integer;
begin
  Assert(not fOrdered, ClassName+': cannot insert object into ordered list');
  i := Count;
  if AfterThis<0 then AfterThis := 0
  else if AfterThis>i then AfterThis := i;
  SetLength(fObjects, i+1);
  if AfterThis<i then
    Move(fObjects[AfterThis], fObjects[AfterThis+1], SizeOf(TObject)*(i-AfterThis));
  fObjects[AfterThis] := Obj;
  DoChanged;
end;

procedure TObjectsList.Clear;
var
  i: Integer;
begin
  i := Count;
  SetLength(fObjects, 0);
  if i>0 then DoChanged;
end;

procedure TObjectsList.FreeAndClear;
var
  i: Integer;
begin
  for i := 0 to Length(fObjects)-1 do fObjects[i].Free;
  Clear;
end;

procedure TObjectsList.Assign(Src: TObjectsList);
var
  i: Integer;
begin
  Assert(not (fOrdered and (not Src.Ordered)), ClassName+': cannot assign unordered list to ordered list');
  i := Length(Src.fObjects);
  SetLength(fObjects, i);
  if i>0 then Move(Src.fObjects[0], fObjects[0], i*SizeOf(TObject));
end;

procedure TObjectsList.Exchange(First, Second: Integer);
var
  Obj: TObject;
begin
  Assert(not fOrdered, ClassName+': cannot exchange objects in ordered list');
  if First<>Second then begin
    CheckIndex(First);
    CheckIndex(Second);
    Obj := fObjects[First-1];
    fObjects[First-1] := fObjects[Second-1];
    fObjects[Second-1] := Obj;
    DoChanged;
  end;
end;

procedure TObjectsList.Remove(Index: Integer);
var
  i: Integer;
begin
  CheckIndex(Index);
  i := Count;
  if Index<i then
    Move(fObjects[Index], fObjects[Index-1], SizeOf(TObject)*(i-Index));
  SetLength(fObjects, i-1);
  DoChanged;
end;

procedure TObjectsList.Remove(Obj: TObject);
var
  i: Integer;
begin
  i := Find(Obj);
  if i>0 then Remove(i);
end;

function TObjectsList.Find(Obj: TObject): Integer;
var
  i, j: Integer;
begin
  if fOrdered and (Count>1) then begin
    i := 0;
    j := Count-1;
    repeat
      Result := (i+j) div 2;
      if Longword(Obj)>Longword(fObjects[Result])
        then i := Result
        else j := Result;
    until (j-i)<=1;
    if fObjects[i]=Obj
      then Result := i+1
    else if fObjects[j]=Obj
      then Result := j+1
      else Result := 0;
  end else begin
    i := 0;
    j := Count;
    Result := 0;
    while (i<j) and (Result=0) do begin
      if fObjects[i]=Obj then Result := i+1;
      Inc(i);
    end;
  end;
end;

function TObjectsList.GetIntersection(WithThis, Output: TObjectsList): Integer;
var
  i: Integer;
  ia: TIntegerDynArray;
begin
  Result := 0;
  for i := 1 to Count do
    if WithThis.Find(Objects[i])>0 then begin
      SetLength(ia, Result+1);
      ia[Result] := i;
      Inc(Result);
    end;

  Output.Clear;
  for i := 0 to Result-1 do
    Output.Add(Objects[ia[i]]);
end;
//------------------------------------------------------------------------------

//======================= Workin with Type Description =========================
procedure GetTypeDescriptionByName(var pTypeDesc: p_typelib_TypeDescription; AName: AnsiString);
var
  pName: prtl_uString;
begin
  pTypeDesc := nil;
  pName := String_to_rtl_uString(AName);
  typelib_typedescription_getByName(pTypeDesc, pName);
  rtl_uString_release(pName);
end;

function GetParamSizeByVarType(AvarType: Word): Integer;
begin
  Result := -1;
  case AvarType of
    varEmpty, varNull: Result := 0;

    varSmallint: Result := SizeOf(SmallInt);
    varInteger: Result := SizeOf(Integer);
    varSingle: Result := SizeOf(Single);
    varDouble: Result := SizeOf(Double);

    varCurrency: Result := SizeOf(Double);
    varDate: Result := SizeOf(Double);

    varOleStr: Result := SizeOf(Pointer);

    varBoolean: Result := SizeOf(Boolean);
    varVariant: Result := SizeOf(Pointer);
    varUnknown: Result := SizeOf(Pointer);

    varShortInt: Result := SizeOf(ShortInt);
    varByte: Result := SizeOf(Byte);
    varWord: Result := SizeOf(Word);
    varLongWord: Result := SizeOf(LongWord);
    varInt64: Result := SizeOf(Int64);

    varStrArg: Result := SizeOf(Pointer);
    varString: Result := SizeOf(Pointer);
    varAny: Result := SizeOf(Pointer);
  end;
end;

//------------------------------------------------------------------------------

//=========================== Working with strings =============================
function String_to_rtl_uString( AStr: WideString ): prtl_uString;
var
  i: sal_Int32;
begin
  Result := nil;
  i := Length(AStr);
  if i>0
    then rtl_uString_newFromStr_WithLength( Result, psal_Unicode(@AStr[1]), i )
    else rtl_uString_newFromStr_WithLength( Result, nil, 0 );
end;

function rtl_uString_to_String(pArtl_uStr: prtl_uString): WideString;
begin
  Result := WideString(psal_Unicode(@pArtl_uStr.buffer));
  SetLength(Result, pArtl_uStr.length);
end;

function compare_rtl_uString_AnsiString( prtlStr: prtl_uString;
                                         Str: AnsiString): boolean;
var
  tmpStr: AnsiString;
begin
  tmpStr := rtl_uString_to_String( prtlStr );
  Result := ( CompareText( Str, tmpStr ) = 0 );
end;
//------------------------------------------------------------------------------

function GetUNOTypeClassByPascalType(const aType: Integer; const aPTR: Pointer): typelib_TypeClass;
begin
  // not standard types
  if ( aType = UNO_Interface.VarType) then
    Result := typelib_TypeClass_INTERFACE
  else if ( aType = UNO_Structure.VarType) then
    Result := typelib_TypeClass_STRUCT
  else
    // standard types
    case aType of
      // Error. Can not translate this types to UNO
      varNull, varAny, varUnknown, varTypeMask,
      varError, varDispatch, varByRef:
        Assert(false, sError_ErrorInParams);

      varVariant: Result := GetUNOTypeClassByPascalType( PVarData(aPTR)^.VType, PVarData(aPTR)^.VPointer );

      varEmpty: Result := typelib_TypeClass_VOID;

      varShortInt: Result := typelib_TypeClass_BYTE;
      varSmallint: Result := typelib_TypeClass_SHORT;
      varWord: Result := typelib_TypeClass_UNSIGNED_SHORT;
      varInteger: Result := typelib_TypeClass_LONG;
      varLongWord: Result := typelib_TypeClass_UNSIGNED_LONG;
      varInt64: Result := typelib_TypeClass_HYPER;

      varSingle: Result := typelib_TypeClass_FLOAT;
      varDouble: Result := typelib_TypeClass_DOUBLE;
      varCurrency: Result := typelib_TypeClass_DOUBLE;
      varDate: Result := typelib_TypeClass_DOUBLE;

      varBoolean: Result := typelib_TypeClass_BOOLEAN;

      varStrArg, varString: Result := typelib_TypeClass_STRING;

      varArray: Result := typelib_TypeClass_SEQUENCE;
    end;
end;

function GetUNOTypeDescriptionReferenceByPascalType(const aPascalType: Integer;
                                                    const aPointer: Pointer): p_typelib_TypeDescriptionReference;
var
  tmp: p_typelib_TypeDescriptionReference;
  i: Integer;
  tmpVar: Variant;
{
  // TODO:
  function CheckSameTypesInArray(aArray: Variant): boolean;
  var
    i, tmpDT: integer;
  begin
    Result := VarArrayHighBound(aArray)<0;
    i := VarArrayLowBound(aArray);

    tmpDT := TVarData(aArray[0])
    while (i<=VarArrayHighBound(aArray)) and (not Result) do begin
      Result := TVarData(aArray[i]).VType<>tmpDT

  end;
}

begin
  Result := nil;
  // not standard types
  if ( aPascalType = UNO_Interface.VarType ) then
    Result := p_typelib_TypeDescriptionReference(TUNO_Pascal_proxy(aPointer).UNOInterfaceTypeDescription)
  else if ( aPascalType = UNO_Structure.VarType ) then
    Result := p_typelib_TypeDescriptionReference(TUNO_Pascal_Struct(aPointer).UNOStructureTypeDescription)
  else if ( aPascalType = UNO_Type.VarType ) then
    Result := TUNO_Pascal_Type(aPointer).UNOType
  else if ( (aPascalType and varArray) <> 0 ) then begin
    // for correct alloc memory for arrays we mast use only Any array
    typelib_static_sequence_type_init( Result, typelib_static_type_getByTypeClass(typelib_TypeClass_ANY)^)
{
    tmpVar := Variant(aPointer^);
    if VarIsNull(tmpVar) or ( VarArrayHighBound( tmpVar, 1 ) < 0 )
      then typelib_static_sequence_type_init( @Result,
                                              typelib_static_type_getByTypeClass(typelib_TypeClass_ANY)^)
      else typelib_static_sequence_type_init( @Result,
                                              GetUNOTypeDescriptionReferenceByPascalType( TVarData(tmpVar[0]).VType,
                                                                                          TVarData(tmpVar[0]).VPointer ));
}
  end else
    // standard types
    case aPascalType of
      // Error
      varNull, varAny, varUnknown, varTypeMask,
      varError, varDispatch, varByRef:
        Assert(false, sError_ErrorInParams);

      varVariant: if VarIsArray(Variant(aPointer^))
                    then Result := typelib_static_type_getByTypeClass(typelib_TypeClass_SEQUENCE)^
                    else Result := GetUNOTypeDescriptionReferenceByPascalType( PVarData(aPointer)^.VType, PVarData(aPointer)^.VPointer );

      varEmpty: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_VOID)^;

      varShortInt: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_BYTE)^;
      varSmallint: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_SHORT)^;
      varWord, varByte: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_UNSIGNED_SHORT)^;
      varInteger: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_LONG)^;
      varLongWord: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_UNSIGNED_LONG)^;
      varInt64: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_HYPER)^;

      varSingle: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_FLOAT)^;
      varDouble: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_DOUBLE)^;
      varCurrency: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_DOUBLE)^;
      varDate: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_DOUBLE)^;

      varBoolean: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_BOOLEAN)^;

      varStrArg, varString, varOleStr: Result := typelib_static_type_getByTypeClass(typelib_TypeClass_STRING)^;

      else
        Result := nil;
    end;
end;

function GetUNOTypeDescriptionByPascalType( const aPascalType: Integer;
                                            const aPointer: Pointer): p_typelib_TypeDescription;
var
  tmp: p_typelib_TypeDescriptionReference;
begin
  Result := nil;
  tmp := GetUNOTypeDescriptionReferenceByPascalType( aPascalType, aPointer );
  typelib_typedescriptionreference_getDescription( Result, tmp );
end;

function CheckReducibility(AUNOType: typelib_TypeClass; AvarType: TVarType): boolean;
begin
  // Any correct for all
  if (AUNOType = typelib_TypeClass_ANY) then
    Result := True
  else if ( AvarType = UNO_Interface.VarType ) then
    Result := (AUNOType=typelib_TypeClass_INTERFACE)
  else if ( AvarType = UNO_Structure.VarType ) then
    Result := (AUNOType=typelib_TypeClass_STRUCT)
  else if ( AvarType = UNO_Type.VarType ) then
    Result := (AUNOType=typelib_TypeClass_TYPE)
  else if ((AvarType and varArray)<>0) then
    Result := (AUNOType=typelib_TypeClass_SEQUENCE) or (AUNOType=typelib_TypeClass_ARRAY)
  else
    case AvarType of
      varUnknown: Result := True;
      // Error
      varNull, varAny, varTypeMask,
      varError, {varUnknown,} varDispatch:
        Result := False;

      // Variant carrect for all
      varVariant, varByRef: Result := True;

      varEmpty: Result := (AUNOType=typelib_TypeClass_VOID);

      varShortInt, varSmallint, varWord, varInteger, varLongWord, varInt64, varByte:
        Result := ((AUNOType>typelib_TypeClass_BOOLEAN) and (AUNOType<=typelib_TypeClass_DOUBLE)) or
                 (AUNOType=typelib_TypeClass_Enum);

      varSingle, varDouble, varCurrency, varDate:
        Result := (AUNOType=typelib_TypeClass_FLOAT) or
                (AUNOType=typelib_TypeClass_DOUBLE);

      varBoolean: Result := (AUNOType=typelib_TypeClass_BOOLEAN);

      varStrArg, varString, varOleStr: Result := (AUNOType=typelib_TypeClass_STRING);

// TODO: test (when its need)?
      varArray:
        Result := (AUNOType=typelib_TypeClass_SEQUENCE) or
                  (AUNOType=typelib_TypeClass_ARRAY);
    end;
end;

//------------------------------------------------------------------------------

function AlignSize(pParamTD: p_typelib_TypeDescription): Integer;
begin
  Result := pParamTD^.nAlignment*Ceil(pParamTD^.nSize/pParamTD^.nAlignment);
//  Result := SizeOf(Pointer)*Ceil(Result/4);
  Result := (Result+(SizeOf(Pointer)-1)) and (not (SizeOf(Pointer)-1));
end;

function NeedPassByRef(pParamTD: p_typelib_TypeDescriptionReference): Boolean;
begin
  Result := ( pParamTD^.eTypeClass = typelib_TypeClass_STRING) or
            ( pParamTD^.eTypeClass = typelib_TypeClass_TYPE) or
            ( pParamTD^.eTypeClass = typelib_TypeClass_SEQUENCE) or
            ( pParamTD^.eTypeClass = typelib_TypeClass_INTERFACE)
            ;
end;

function NeedNoCopy(pParamTD: p_typelib_TypeDescriptionReference): Boolean;
begin
  Result := ( pParamTD^.eTypeClass = typelib_TypeClass_TYPE ) or
            ( pParamTD^.eTypeClass = typelib_TypeClass_STRUCT ) or
            ( pParamTD^.eTypeClass = typelib_TypeClass_SEQUENCE ) or
            ( pParamTD^.eTypeClass = typelib_TypeClass_INTERFACE ) or
            ( pParamTD^.eTypeClass = typelib_TypeClass_SERVICE );
end;

function HasWrapper( ATypeClass: typelib_TypeClass): Boolean;
begin
  Result := ( ATypeClass = typelib_TypeClass_TYPE ) or
            ( ATypeClass = typelib_TypeClass_STRUCT ) or
            ( ATypeClass = typelib_TypeClass_INTERFACE )
            ;
end;

function NeedGetMemForConvert(ATD: p_typelib_TypeDescriptionReference): boolean;
begin       // memory gets by own mem manager
  Result := ( not HasWrapper(ATD^.eTypeClass) ) and
            // memory gets by UNO
            ( ATD^.eTypeClass <> typelib_TypeClass_STRING ) and
            ( ATD^.eTypeClass <> typelib_TypeClass_SEQUENCE );
end;

function EndTypeClass(pTD: p_typelib_TypeDescription; pData: Pointer): typelib_TypeClass;
var
  tmpData: Pointer;
begin
  Result := pTD^.eTypeClass;
  while Result = typelib_TypeClass_ANY do begin
    if NeedPassByRef(p_uno_Any(pData)^.pType)
      then tmpData := Pointer(p_uno_Any(pData)^.pData^)
      else tmpData := p_uno_Any(pData)^.pData;
    Result := p_uno_Any(pData)^.pType^.eTypeClass;
  end;
end;


function DestroyData_and_FreeMem(var ADataPtr: Pointer; ADT: p_typelib_TypeDescriptionReference): Boolean;
var
  ptmpTD:  p_typelib_TypeDescription;
  tmpArrayMember,tmpArrayMemberTrue: Pointer;
  i: integer;
begin
  if not HasWrapper( ADT^.eTypeClass ) then
    case ADT^.eTypeClass of
      typelib_TypeClass_ANY: begin
//        DestroyData_and_FreeMem( p_uno_Any(ADataPtr).pData, p_uno_Any(ADataPtr).pType );
//        if NeedPassByRef(p_uno_Any(ADataPtr).pType) and NeedGetMemForConvert(p_uno_Any(ADataPtr).pType)
//          then FreeMem(p_uno_Any(ADataPtr).pData);
        uno_destructData(ADataPtr, ADT^.pType , nil);
//        uno_any_destruct(ADataPtr, nil);
//        FreeMem(ADataPtr);
//        ADataPtr := nil;
      end;

      typelib_TypeClass_SEQUENCE: begin
      end;

{
      typelib_TypeClass_SEQUENCE: begin
        ptmpTD := nil;
        typelib_typedescriptionreference_getDescription( ptmpTD,  p_typelib_IndirectTypeDescription(ADT).pType);
        tmpArrayMember := Pointer(@psal_Sequence( ADataPtr )^.elements);

        for i := 0 to ( psal_Sequence(PPointer(ADataPtr)^)^.nElements-1 ) do begin
          if NeedPassByRef( p_typelib_TypeDescriptionReference(ptmpTD) )
            then tmpArrayMemberTrue := PPointer(tmpArrayMember)^
            else tmpArrayMemberTrue := tmpArrayMember;
          DestroyData_and_FreeMem( tmpArrayMemberTrue, p_typelib_TypeDescriptionReference(ptmpTD) );
          Inc( Integer(tmpArrayMember), ptmpTD^.nSize );
        end;
        uno_destructData(ADataPtr, ADT^.pType , nil);
        typelib_typedescription_release(ptmpTD);
      end;
}
      typelib_TypeClass_STRING: begin
        rtl_uString_release(ADataPtr);
      end;

      else
        uno_destructData(ADataPtr, ADT^.pType , nil);
    end;
{
  if not HasWrapper( ADT^.eTypeClass ) and
     (ADT^.eTypeClass <> typelib_TypeClass_ANY) and
     (ADT^.eTypeClass <> typelib_TypeClass_SEQUENCE)
    then uno_destructData(ADataPtr, ADT^.pType , nil);

  if (ADT^.eTypeClass = typelib_TypeClass_ANY) then begin
    DestroyData_and_FreeMem( p_uno_Any(ADataPtr).pData, p_uno_Any(ADataPtr).pType );
    FreeMem(ADataPtr);

    if NeedPassByRef(p_uno_Any(ADataPtr)^.pType) then begin
      uno_destructData(p_uno_Any(ADataPtr)^.pData, p_uno_Any(ADataPtr)^.pType^.pType, nil);
      FreeMem(PPointer(p_uno_Any(ADataPtr)^.pData)^);
    end;

//    uno_destructData(ADataPtr, ADT^.pType , nil);
  end
}
end;

procedure PascalParam2UNOParam( pPascalParam: Pointer;
                                APascalParamType: Integer;
                                pUNOParamTD: p_typelib_TypeDescriptionReference;
                                var pOutUNOParam: Pointer;
                                var AUNOSize: Integer;
                                NeverMove: Boolean = False);
var
//------------------------------------------
  // counter array members
  m: Integer;
  // count of elements in varArray
  fMembersCount: Integer;
  // array of pointers on elements (need for construct uno array)
  tmpPrtArray: array of Pointer;
  //
  tmpRefOnArray: Pointer;
  tmpElemSizeWithAlig: Integer;
  tmpSeqSizeWithAlig: Integer;

  tmpVarArrayMember: Variant;
  pVarArrayTD: p_typelib_IndirectTypeDescription;

//------------------------------------------
  tmp_prtl_uString: prtl_uString;
//------------------------------------------
  tmp_pUNO_Any: p_uno_any;
  tmp_pUNO_Any_Data: Pointer;
  //
  tmp_ppUNO_Any_Data: Pointer;
  tmp_pUNO_Any_TD: p_typelib_TypeDescription;
//------------------------------------------
  // bibbs fo recursion
  tmpSize: Integer;
  tmpPtr, tmpPtr1: Pointer;
  tmpDouble: Double;
  tmpSingle: Single;

  fNeedGetMemForConvert: Boolean;

  procedure MakeOut(_pOutUNOParam: Pointer; _AUNOSize: Integer; NeedMove: Boolean = False);
  begin
    if NeedMove and (not NeverMove)
      then move( _pOutUNOParam^, pOutUNOParam^, _AUNOSize)
      else pOutUNOParam := _pOutUNOParam;
    AUNOSize := _AUNOSize;
  end;

begin
  // Check reducibility needed params with resived
  Assert( CheckReducibility(pUNOParamTD^.eTypeClass, APascalParamType), sError_ErrorInParamType);

  // for Any
  if (pUNOParamTD^.eTypeClass = typelib_TypeClass_ANY) then begin
    // get Type Desription for packing data
    tmp_pUNO_Any_TD := GetUNOTypeDescriptionByPascalType( APascalParamType, pPascalParam );
    fNeedGetMemForConvert := NeedGetMemForConvert(p_typelib_TypeDescriptionReference(tmp_pUNO_Any_TD));

    // Get memory for Any Data
    if fNeedGetMemForConvert
      then GetMem( tmp_pUNO_Any_Data, AlignSize(tmp_pUNO_Any_TD) )
      else tmp_pUNO_Any_Data := nil;

    // translate Pascal parameter in UNO parameter
    PascalParam2UNOParam( pPascalParam,
                          APascalParamType,
                          p_typelib_TypeDescriptionReference(tmp_pUNO_Any_TD),
                          tmp_pUNO_Any_Data,
                          tmpSize);

    // processing for not standard types
    if NeedPassByRef(p_typelib_TypeDescriptionReference(tmp_pUNO_Any_TD))
      then tmp_ppUNO_Any_Data := @tmp_pUNO_Any_Data
      else tmp_ppUNO_Any_Data := tmp_pUNO_Any_Data;

    if not Assigned(pOutUNOParam)
      then GetMem( tmp_pUNO_Any, SizeOf(uno_Any) )
      else tmp_pUNO_Any := pOutUNOParam;
    // create Any
    uno_any_construct(tmp_pUNO_Any,
                      tmp_ppUNO_Any_Data,
                      tmp_pUNO_Any_TD,
                      nil);

    DestroyData_and_FreeMem(tmp_pUNO_Any_Data, tmp_pUNO_Any_TD.pWeakRef);
    if fNeedGetMemForConvert then FreeMem(tmp_pUNO_Any_Data);
    // Get result
    MakeOut(tmp_pUNO_Any, SizeOf(uno_Any), False);
    // free TD
    typelib_typedescription_release(tmp_pUNO_Any_TD);
  end else if ( APascalParamType = UNO_Interface.VarType) then
    // TODO: mapping if need
    MakeOut( TUNO_Pascal_proxy({PVarData(}pPascalParam{)^.VPointer}).UNOInterface,
             SizeOf(Pointer) )
  else if ( APascalParamType = UNO_Structure.VarType) then begin
    MakeOut( TUNO_Pascal_Struct(pPascalParam).UNOStructurePTR,
             TUNO_Pascal_Struct(pPascalParam).StructureSize,
             Assigned(pOutUNOParam) );
  end else if ( APascalParamType = UNO_Type.VarType) then
    MakeOut( TUNO_Pascal_Type(pPascalParam).UNOType , SizeOf(Pointer) )
  else if ( (APascalParamType and varArray) <> 0 ) then begin
{
    if VarIsArray(Variant(pPascalParam^)) then begin
      // get TypeDesc on Array
      pVarArrayTD := nil;
      typelib_typedescriptionreference_getDescription(p_typelib_typedescription(pVarArrayTD), pUNOParamTD);
      // check
      Assert( Assigned(pVarArrayTD) and CheckReducibility(pUNOParamTD.eTypeClass, varArray) , sError_ErrorInParamType);
      // Calculate array elements count
      fMembersCount := ( VarArrayHighBound(Variant(pPascalParam^), 1) -
                         VarArrayLowBound(Variant(pPascalParam^), 1) ) + 1;
      // TODO: check by elements count

      if NeedPassByRef( pVarArrayTD^.pType )
        then tmpElemSizeWithAlig := SizeOf(Pointer)
        else tmpElemSizeWithAlig := AlignSize(pVarArrayTD^.pType^.pType);

      // create uno sequence
      uno_type_sequence_construct(puno_Sequence(pOutUNOParam), pUNOParamTD, nil, fMembersCount, nil);
      tmpRefOnArray := @puno_Sequence(pOutUNOParam).elements;
      SetLength(tmpPrtArray, fMembersCount);
      // translate all array elements
      for m := 0 to (fMembersCount-1) do begin
        tmpVarArrayMember := Variant(pPascalParam^)[m];
        //
        if VarIsArray(tmpVarArrayMember)
          then tmpPtr := @tmpVarArrayMember
          else tmpPtr := TVarData(tmpVarArrayMember).VPointer;
        //
        PascalParam2UNOParam( tmpPtr,
                              VarType(tmpVarArrayMember),
                              pVarArrayTD^.pType,
                              tmpPrtArray[m],
                              tmpSize);

        if NeedPassByRef( pVarArrayTD^.pType )
          then Move( tmpPrtArray[m], Pointer(Integer(tmpRefOnArray)+Integer(m*tmpElemSizeWithAlig))^, tmpElemSizeWithAlig )
          else Move( tmpPrtArray[m]^, Pointer(Integer(tmpRefOnArray)+Integer(m*tmpElemSizeWithAlig))^, tmpElemSizeWithAlig );
      end;

      AUNOSize := SizeOf(psal_Sequence);
      typelib_typedescription_release(p_typelib_TypeDescription(pVarArrayTD));
    end
}
        if VarIsArray(Variant(pPascalParam^)) then begin
          // get TypeDesc on array
          pVarArrayTD := nil;
          typelib_typedescriptionreference_getDescription(p_typelib_typedescription(pVarArrayTD), pUNOParamTD);
          //
          Assert( Assigned(pVarArrayTD) and
                  CheckReducibility(pUNOParamTD.eTypeClass, varArray),
                  sError_ErrorInParamType );

          // Calculate array elements count
          fMembersCount := ( VarArrayHighBound(Variant(pPascalParam^), 1) -
                           VarArrayLowBound(Variant(pPascalParam^), 1) ) + 1;
         // TODO: check by elements count

          if NeedPassByRef( pVarArrayTD^.pType )
            then tmpElemSizeWithAlig := SizeOf(Pointer)
            else tmpElemSizeWithAlig := AlignSize(pVarArrayTD^.pType^.pType);

          uno_type_sequence_construct( puno_Sequence(pOutUNOParam), pUNOParamTD,
                                       nil, fMembersCount, nil );
          tmpRefOnArray := @puno_Sequence(pOutUNOParam).elements;

          SetLength(tmpPrtArray, fMembersCount);
          for m := 0 to (fMembersCount-1) do begin
            tmpVarArrayMember := Variant(pPascalParam^)[m];

            if VarIsArray(tmpVarArrayMember)
              then tmpPtr := @tmpVarArrayMember
              else case TVarData(tmpVarArrayMember).VType of
                varSmallInt,
                varInteger,
                varSingle,
                varDouble,
                varCurrency,
                varDate,
                varError,
                varBoolean,
                varShortInt,
                varByte,
                varWord,
                varLongWord,
                varInt64: tmpPtr := @TVarData(tmpVarArrayMember).VBoolean;
                else
                  tmpPtr := TVarData(tmpVarArrayMember).VPointer;
              end;

            tmpPtr1 := Pointer(Integer(tmpRefOnArray) + Integer(m*tmpElemSizeWithAlig));

            if NeedPassByRef( pVarArrayTD^.pType )
              then PascalParam2UNOParam( tmpPtr, VarType(tmpVarArrayMember),
                                         pVarArrayTD^.pType, PPointer(tmpPtr1)^, tmpSize)
              else PascalParam2UNOParam( tmpPtr, VarType(tmpVarArrayMember),
                                         pVarArrayTD^.pType, tmpPtr1, tmpSize)
          end;

          AUNOSize := SizeOf(psal_Sequence);
          typelib_typedescription_release(p_typelib_TypeDescription(pVarArrayTD));
        end;
  end else
    case APascalParamType of
      // can not translate this types to uno
      varEmpty, varNull, varAny,
      varError, varDispatch:
        Assert(false, 'Error. Param with type '+ VarTypeAsText(APascalParamType) +' can not be passed to UNO');

      varBoolean: MakeOut(pPascalParam, SizeOf(sal_Bool),true);

      // int types
      varByte, varShortInt, varSmallint, varWord, varInteger, varLongWord, varInt64: begin
        case pUNOParamTD.eTypeClass of
          typelib_TypeClass_BYTE: MakeOut(pPascalParam, SizeOf(Byte), True);
          typelib_TypeClass_SHORT: MakeOut(pPascalParam, SizeOf(Smallint), True);
          typelib_TypeClass_UNSIGNED_SHORT: MakeOut(pPascalParam, SizeOf(Word), True);
          typelib_TypeClass_LONG: MakeOut(pPascalParam, SizeOf(Integer), True);
          typelib_TypeClass_UNSIGNED_LONG: MakeOut(pPascalParam, SizeOf(LongWord), True);
          typelib_TypeClass_HYPER, typelib_TypeClass_UNSIGNED_HYPER: MakeOut(pPascalParam, SizeOf(Int64), True);
          typelib_TypeClass_FLOAT: MakeOut(pPascalParam, SizeOf(Single), True);
          typelib_TypeClass_DOUBLE: MakeOut(pPascalParam, SizeOf(Double), True);
          typelib_TypeClass_Enum: MakeOut(pPascalParam, SizeOf(LongWord), True);
          else Assert(false, sError_ErrorInParamType);
        end;
      end;

      // float types
      varSingle, varDouble, varCurrency, varDate: begin
        // reduction
        case APascalParamType of
          varSingle: tmpDouble := Single(pPascalParam^);
          varDouble: tmpDouble := Double(pPascalParam^);
          varCurrency: tmpDouble := Currency(pPascalParam^);
          varDate: tmpDouble := TDateTime(pPascalParam^);
        end;
        case pUNOParamTD.eTypeClass of
          typelib_TypeClass_FLOAT: begin
            tmpSingle := tmpDouble;
            MakeOut(@tmpSingle, SizeOf(Single), True);
          end;
          typelib_TypeClass_DOUBLE: MakeOut(@tmpDouble, SizeOf(Double), True);
          else Assert(false, sError_ErrorInParamType);
        end;
      end;

      // string types
      varStrArg: MakeOut(String_to_rtl_uString(System.Copy( PString(pPascalParam)^, 1, MaxInt)), SizeOf(prtl_uString));
      varString: MakeOut(String_to_rtl_uString(String(pChar(pPascalParam))), SizeOf(prtl_uString));
      varOleStr: MakeOut(String_to_rtl_uString(pWideChar(pPascalParam)), SizeOf(prtl_uString));

      // array
      varVariant: begin
        if VarIsArray(Variant(pPascalParam^)) then begin
          // get TypeDesc on array
          pVarArrayTD := nil;
          typelib_typedescriptionreference_getDescription(p_typelib_typedescription(pVarArrayTD), pUNOParamTD);
          //
          Assert( Assigned(pVarArrayTD) and
                  CheckReducibility(pUNOParamTD.eTypeClass, varArray),
                  sError_ErrorInParamType );

          // Calculate array elements count
          fMembersCount := ( VarArrayHighBound(Variant(pPascalParam^), 1) -
                           VarArrayLowBound(Variant(pPascalParam^), 1) ) + 1;
         // TODO: check by elements count

          if NeedPassByRef( pVarArrayTD^.pType )
            then tmpElemSizeWithAlig := SizeOf(Pointer)
            else tmpElemSizeWithAlig := AlignSize(pVarArrayTD^.pType^.pType);

          uno_type_sequence_construct(puno_Sequence(pOutUNOParam), pUNOParamTD, nil, fMembersCount, nil);

          tmpRefOnArray := @puno_Sequence(pOutUNOParam).elements;

          SetLength(tmpPrtArray, fMembersCount);
          for m := 0 to (fMembersCount-1) do begin
            tmpVarArrayMember := Variant(pPascalParam^)[m];

            if VarIsArray(tmpVarArrayMember)
              then tmpPtr := @tmpVarArrayMember
              else case TVarData(tmpVarArrayMember).VType of
                varSmallInt,
                varInteger,
                varSingle,
                varDouble,
                varCurrency,
                varDate,
                varError,
                varBoolean,
                varShortInt,
                varByte,
                varWord,
                varLongWord,
                varInt64: tmpPtr := @TVarData(tmpVarArrayMember).VBoolean;
                else
                  tmpPtr := TVarData(tmpVarArrayMember).VPointer;
              end;

            tmpPtr1 := Pointer(Integer(tmpRefOnArray) + Integer(m*tmpElemSizeWithAlig));

            if NeedPassByRef( pVarArrayTD^.pType )
              then PascalParam2UNOParam( tmpPtr, VarType(tmpVarArrayMember),
                                         pVarArrayTD^.pType, PPointer(tmpPtr1)^, tmpSize)
              else PascalParam2UNOParam( tmpPtr, VarType(tmpVarArrayMember),
                                         pVarArrayTD^.pType, tmpPtr1, tmpSize)
          end;

          AUNOSize := SizeOf(psal_Sequence);
          typelib_typedescription_release(p_typelib_TypeDescription(pVarArrayTD));
        end else begin
          case pUNOParamTD.eTypeClass of
            typelib_TypeClass_INTERFACE, typelib_TypeClass_SERVICE: begin
              // TODO: mapping (if need)
              MakeOut(TUNO_Pascal_proxy(PVarData(pPascalParam)^.VPointer).UNOInterface, SizeOf(Pointer));
            end;
            typelib_TypeClass_TYPE: begin
              // suppose, that variant contains pointer on p_typelib_typedescriptionReference
              MakeOut(TUNO_Pascal_Type(PVarData(pPascalParam)^.VPointer).UNOType, SizeOf(Pointer));
            end;
            typelib_TypeClass_STRUCT: begin
              MakeOut(TUNO_Pascal_Struct(PVarData(pPascalParam)^.VPointer).UNOStructurePTR, SizeOf(Pointer));
            end;

            else begin
              pOutUNOParam := nil;
              PascalParam2UNOParam( PVarData(pPascalParam)^.VPointer,
                                    PVarData(pPascalParam)^.VType,
                                    PUNOParamTD, pOutUNOParam, AUNOSize);
            end;
          end;
        end;
      end; // varVariant:

      varByRef: begin
        tmpPtr := nil;
        PascalParam2UNOParam( PVarData(pPascalParam^)^.VPointer,
                              PVarData(pPascalParam^)^.VType,
                              pUNOParamTD, Pointer(tmpPtr),
                              Integer(tmpSize) );
      end;

    end; // case LArgType of
end;

procedure UnoParam2PascalParam( AUnoParamPointer: Pointer;
                                AUnoParamTypeDesc: p_typelib_TypeDescriptionReference;
                                var APascalPointer: Pointer;
                                var APascalParam: Variant;
                                Reserved: Pointer = nil);
var
  tmpArrayTD: p_typelib_ArrayTypeDescription;
  ArrayDimension: array of Integer;
  pSeqMemberTD: p_typelib_TypeDescription;

  tmpPtr: Pointer;
  tmpSize: Integer;

  tmpVar: Variant;
  tmpArrayMember: Pointer;
  tmpArrayMemberToConvert: Pointer;


  tmpAnyData: Pointer;

  i: Integer;
  tmpStr: String;

  procedure MakeOut(P: Pointer; V: Variant);
  begin
    APascalPointer := P;
    APascalParam := V;
  end;

begin
  APascalPointer := nil;
  tmpPtr := nil;
  VarClear(APascalParam);
  case AUnoParamTypeDesc.eTypeClass of
    typelib_TypeClass_VOID: begin
                              VarClear(APascalParam);
                              TVarData(tmpVar).VPointer := nil;
                              TVarData(tmpVar).VType := varEmpty;
                              MakeOut(nil, tmpVar);
                            end;
    typelib_TypeClass_CHAR: MakeOut(AUnoParamPointer,Char(AUnoParamPointer^));
    typelib_TypeClass_BOOLEAN: MakeOut(AUnoParamPointer,sal_Bool(AUnoParamPointer^));
    typelib_TypeClass_BYTE: MakeOut(AUnoParamPointer,Byte(AUnoParamPointer^));
    typelib_TypeClass_SHORT: MakeOut(AUnoParamPointer,SmallInt(AUnoParamPointer^));
    typelib_TypeClass_UNSIGNED_SHORT: MakeOut(AUnoParamPointer,Word(AUnoParamPointer^));
    typelib_TypeClass_LONG: MakeOut(AUnoParamPointer,Integer(AUnoParamPointer^));
    typelib_TypeClass_UNSIGNED_LONG: MakeOut(AUnoParamPointer,LongWord(AUnoParamPointer^));
    typelib_TypeClass_HYPER: MakeOut(AUnoParamPointer,Int64(AUnoParamPointer^));
    typelib_TypeClass_UNSIGNED_HYPER: MakeOut(AUnoParamPointer,Int64(AUnoParamPointer^));
    typelib_TypeClass_FLOAT: MakeOut(AUnoParamPointer,Single(AUnoParamPointer^));
    typelib_TypeClass_DOUBLE: MakeOut(AUnoParamPointer,Double(AUnoParamPointer^));
    typelib_TypeClass_STRING: begin
      tmpStr := rtl_uString_to_String(prtl_uString(AUnoParamPointer));
      MakeOut(@tmpStr,tmpStr);
    end;

    typelib_TypeClass_INTERFACE: begin
      if Assigned(AUnoParamPointer) then with TVarData(APascalParam) do begin
        VType := UNO_Interface.VarType;
        VPointer := Pascal_Bridge.GetProxyByUnoInterface( AUnoParamPointer,
                                              p_typelib_InterfaceTypeDescription(AUnoParamTypeDesc));
      end;
      APascalPointer := @APascalParam;
    end;

    typelib_TypeClass_SEQUENCE: begin
      // get Type Desc of Sequence element
      typelib_typedescriptionreference_getDescription(pSeqMemberTD,  p_typelib_IndirectTypeDescription(AUnoParamTypeDesc).pType);
      // create Var Array
      APascalParam := VarArrayCreate( [0, psal_Sequence(AUnoParamPointer)^.nElements-1], varVariant );
      // copy Sequence elemetnts in VarArray
      tmpArrayMember := Pointer(@psal_Sequence( AUnoParamPointer )^.elements);

      for i := 0 to ( psal_Sequence(AUnoParamPointer)^.nElements-1 ) do begin
        if NeedPassByRef( p_typelib_TypeDescriptionReference(pSeqMemberTD) )
          then tmpArrayMemberToConvert := PPointer(tmpArrayMember)^
          else tmpArrayMemberToConvert := tmpArrayMember;

        UnoParam2PascalParam( tmpArrayMemberToConvert, p_typelib_TypeDescriptionReference(pSeqMemberTD), tmpPtr, tmpVar );
        APascalParam[i] := tmpVar;
        Inc( Integer(tmpArrayMember), pSeqMemberTD^.nSize );
      end;
      APascalPointer := @APascalParam;
      typelib_typedescription_release(pSeqMemberTD);
    end;

    typelib_TypeClass_ANY: begin
      // TODO: Check poiner on pointer translation
      if NeedPassByRef(p_uno_Any(AUnoParamPointer)^.pType)
        then tmpAnyData := Pointer(p_uno_Any(AUnoParamPointer)^.pData^)
        else tmpAnyData := p_uno_Any(AUnoParamPointer)^.pData;

      UnoParam2PascalParam( tmpAnyData,
                            p_uno_Any(AUnoParamPointer)^.pType,
                            APascalPointer, APascalParam);
    end;

    typelib_TypeClass_TYPE: begin
      if Assigned(AUnoParamPointer) then with TVarData(APascalParam) do begin
        VType := UNO_Type.VarType;
        VPointer := TUNO_Pascal_Type.CreateByUNOType(AUnoParamPointer);
      end;
      APascalPointer := @APascalParam;
    end;

    typelib_TypeClass_STRUCT: begin
      if Assigned(AUnoParamPointer) then with TVarData(APascalParam) do begin
        VType := UNO_Structure.VarType;
        VPointer := TUNO_Pascal_Struct.CreateByUNOStruct(AUnoParamPointer, AUnoParamTypeDesc);
      end;
      APascalPointer := @APascalParam;
    end;
  end;
end;

function GetMemberTypeDescByName(pInterfaceTypeDescription: p_typelib_InterfaceTypeDescription;
                                 aMemberName: AnsiString): p_typelib_InterfaceMemberTypeDescription;
const
  sFullNameDivider = ':@';
  sNameDivider = '::';

var
  i: Integer;
  tmpMember: pp_typelib_TypeDescriptionReference;
  tmpStr, tmpName: AnsiString;
  srtPos, endPos: Integer;
begin
  i := 0;
  Result := nil;
  tmpMember := pInterfaceTypeDescription^.ppAllMembers;
  while (not Assigned(Result)) and ( i<pInterfaceTypeDescription^.nAllMembers ) do begin
    // getting full name of method ( <Inteface name>::<Method name> )
    tmpStr := rtl_uString_to_String(tmpMember^.pTypeName);
    // getting short name of method ( <Method name> )
    srtPos := Pos(sNameDivider, tmpStr);
    if (srtPos <> 0) then srtPos := srtPos+Length(sNameDivider);
    endPos := Pos(sFullNameDivider, tmpStr);
    if (endPos = 0) then endPos := Length(tmpStr)+1;
    tmpName := AnsiUpperCase(Copy( tmpStr, srtPos, endPos-srtPos ));
    // checking
    if AnsiCompareStr(tmpName, aMemberName)=0
      then TYPELIB_DANGER_GET( p_typelib_TypeDescription(Result), tmpMember^ );

    inc(tmpMember);
    inc(i);
  end;
end;

procedure FreeMemberTypeDesc(pInterfaceMemberTypeDescription: p_typelib_InterfaceMemberTypeDescription);
begin
  TYPELIB_DANGER_RELEASE(p_typelib_TypeDescription(p_typelib_TypeDescription(pInterfaceMemberTypeDescription)));
end;

end.
