unit uno_uno_environment;

interface

{$I uno_common.inc}

uses
   uno_sal_config
  ,uno_sal_types
  ,uno_rtl_ustring
  ,uno_typelib_typedescription;

type
   p_uno_ExtEnvironment = ^_uno_ExtEnvironment;
   pp_uno_ExtEnvironment = ^p_uno_ExtEnvironment;

   p_uno_Environment = ^_uno_Environment;
   pp_uno_Environment = ^p_uno_Environment;
   ppp_uno_Environment = ^pp_uno_Environment;

//------------------------------------------------------------------------------

  // The binary specification of an UNO environment.
  _uno_Environment = record
    // reserved for future use (0 if not used)
    pReserved: Pointer;
    // type name of environment
    pTypeName: prtl_uString;
    // free context pointer to be used for specific classes of environments (e.g., a jvm pointer)
    pContext: Pointer;
    // pointer to extended environment (interface registration functionality), if supported
    pExtEnv: p_uno_ExtEnvironment;

    // Acquires this environment. @param pEnv this environment
    acquire: procedure( pEnv: p_uno_Environment ); cdecl;

    // Releases this environment; last release of environment will revoke the environment from
    //  runtime.
    //  @param pEnv this environment
    release: procedure( pEnv: p_uno_Environment ); cdecl;

    // Acquires this environment weakly.  You can only harden a weakly held environment if it
    //  is still acquired hard (acquire()).
    //  @param pEnv this environment
    acquireWeak: procedure( pEnv: p_uno_Environment ); cdecl;

    // Releases this environment weakly in correspondence to acquireWeak().
    //  @param pEnv this environment
    releaseWeak: procedure( pEnv: p_uno_Environment ); cdecl;

    // Makes hard reference out of weak referenced environment. You can only harden a weakly
    //  held environment if it is still acquired hard (acquire()).
    //  @param ppHardEnv inout hard referenced environment (has to be released via release())
    //  @param pEnv environment (may be weak referenced) *)
    harden: procedure( var pHardEnv: p_uno_Environment; pEnv: p_uno_Environment ); cdecl;

    // Call this function to EXPLICITLY dispose this environment (e.g., release all
    //  interfaces). You may want to call this function before shutting down due to a runtime error.
    //  @param pEnv this environment
    dispose: procedure( pEnv: p_uno_Environment ); cdecl;

    // ===== the following part will be late initialized by a matching bridge =====
    // ===== and is NOT for public use.                      =====
    // CALLBACK function pointer: Disposing callback function pointer that can be set to get
    //                 signalled before the environment is destroyed.
    //  @param pEnv environment that is being disposed
    environmentDisposing: procedure( pEnv: p_uno_Environment ); cdecl;
  end;
  uno_Environment = _uno_Environment;

//------------------------------------------------------------------------------

  // Generic function pointer declaration to free a proxy object if it is not needed by the
  //  environment anymore.
  // Any proxy object must register itself on first acquire() call and revoke itself on last
  //  release() call. This can happen several times because the environment caches proxy objects
  //  until the environment explicitly frees the proxy object calling this function.
  // @param pEnv environment
  // @param pProxy proxy pointer
  uno_freeProxyFunc = procedure( pEnv: p_uno_ExtEnvironment; pProxy: Pointer );  cdecl;

  // Generic function pointer declaration to allocate memory. Used with getRegisteredInterfaces().
  // @param nBytes amount of memory in bytes
  // @return pointer to allocated memory
  uno_memAlloc = function( nBytes: sal_Size): Pointer; cdecl;

//------------------------------------------------------------------------------

  // The binary specification of an UNO environment supporting interface registration.
  _uno_ExtEnvironment = record
    // inherits all members of an uno_Environment
    aBase: uno_Environment;

    // Registers an interface of this environment.
    //  @param pEnv   this environment
    //  @param ppInterface inout parameter of interface to be registered
    //  @param pOId   object id of interface
    //  @param pTypeDescr type description of interface
    registerInterface: procedure(pEnv: p_uno_ExtEnvironment; var pInterface: Pointer;
                                  pOId: prtl_uString; pTypeDescr: p_typelib_InterfaceTypeDescription ); cdecl;

    // Registers a proxy interface of this environment that can be reanimated and is freed
    //  explicitly by this environment.
    //  @param pEnv      this environment
    //  @param ppInterface  inout parameter of interface to be registered
    //  @param freeProxy  function to free proxy object
    //  @param pOId      object id of interface
    //  @param pTypeDescr  type description of interface
    registerProxyInterface: procedure( pEnv: p_uno_ExtEnvironment; var pProxy: Pointer;
                      freeProxy: uno_freeProxyFunc;
                      pOId: prtl_uString;
                      pTypeDescr: p_typelib_InterfaceTypeDescription ); cdecl;

    // Revokes an interface from this environment. You have to revoke any interface that has
    //  been registered via this method.
    //  @param pEnv      this environment
    //  @param pInterface  interface to be revoked
    revokeInterface: procedure( pEnv: p_uno_ExtEnvironment; pInterface: Pointer ); cdecl;

    // Provides the object id of a given interface.
    //  @param ppOut    inout oid
    //  @param pInterface  interface of object
    getObjectIdentifier: procedure( pEnv: p_uno_ExtEnvironment; var pOId: prtl_uString; pInterface: Pointer ); cdecl;

    // Retrieves an interface identified by its object id and type from this environment.
    //  Interfaces are retrieved in the same order as they are registered.
    //  @param pEnv      this environment
    //  @param ppInterface  inout parameter for the registered interface; (0) if none was found
    //  @param pOId      object id of interface to be retrieved
    //  @param pTypeDescr  type description of interface to be retrieved
    getRegisteredInterface: procedure(pEnv: p_uno_ExtEnvironment; var pInterface: Pointer;
                                       pOId: prtl_uString; pTypeDescr: p_typelib_InterfaceTypeDescription ); cdecl;

    // Returns all currently registered interfaces of this environment. The memory block
    //  allocated might be slightly larger than ( *pnLen * sizeof(void * )).
    //  @param pEnv      this environment
    //  @param pppInterfaces out param; pointer to array of interface pointers
    //  @param pnLen    out param; length of array
    //  @param memAlloc    function for allocating memory that is passed back
    getRegisteredInterfaces: procedure(pEnv: p_uno_ExtEnvironment; var ppInterface: PPointer;
                      var nLen: sal_Int32; memAlloc: uno_memAlloc); cdecl;

    // ===== the following part will be late initialized by a matching bridge =====

    // Computes an object id of the given interface; is called by the environment implementation.
    //  @param pEnv      corresponding environment
    //  @param ppOId    out param: computed id
    //  @param pInterface  an interface
    computeObjectIdentifier: procedure( pEnv: p_uno_ExtEnvironment; var pOId: prtl_uString; pInterface: Pointer ); cdecl;

    // Function to acquire an interface.
    //  @param pEnv      corresponding environment
    //  @param pInterface  an interface
    acquireInterface: procedure( pEnv: p_uno_ExtEnvironment; pInterface: Pointer ); cdecl;

    // Function to release an interface.
    //  @param pEnv      corresponding environment
    //  @param pInterface  an interface
    releaseInterface: procedure( pEnv: p_uno_ExtEnvironment; pInterface: Pointer ); cdecl;
  end;
  uno_ExtEnvironment = _uno_ExtEnvironment;

//------------------------------------------------------------------------------

  // Function exported by some bridge library providing acquireInterface(), releaseInterface();
  //  may set a disposing callback.
  // @param pEnv environment to be initialized
  uno_initEnvironmentFunc = procedure( pEnv: p_uno_ExtEnvironment ); cdecl;

const
  UNO_INIT_ENVIRONMENT = 'uno_initEnvironment';

  // Gets a specific environment. If the specified environment does not exist, then a default one
  //  is created and registered. The environment revokes itself on last release() call.
  // @param ppEnv  inout parameter of environment; given environment will be released
  // @param pEnvTypeName type name of environment
  // @param pContext  some context pointer (e.g., to distinguish java vm; set 0 if not needed)
  procedure uno_getEnvironment( var pEnv: p_uno_Environment; pEnvTypeName: prtl_uString; pContext: Pointer); cdecl;

  // Gets all specified environments. Caller has to release returned environments and free allocated
  //  memory.
  // @param pppEnvs  out param; pointer to array of environments
  // @param pnLen  out param; length of array
  // @param memAlloc  function for allocating memory that is passed back
  // @param pEnvTypeName type name of environments; 0 defaults to all
  procedure uno_getRegisteredEnvironments( var ppEnvs: pp_uno_Environment; var nLen: sal_Int32;
                        memAlloc: uno_memAlloc; pEnvTypeName: prtl_uString ); cdecl;

  // Creates an environment. The new environment is anonymous (NOT publicly registered/ accessible).
  // @param ppEnv  out parameter of environment; given environment will be released
  // @param pEnvTypeName name of environment
  // @param pContext  context pointer (e.g., to distinguish java vm); set 0 if not needed
  procedure uno_createEnvironment( var pEnv: p_uno_Environment; pEnvTypeName: prtl_uString; pContext: Pointer ); cdecl;

  // Dumps out environment information, i.e. registered interfaces.
  // @param stream  output stream (FILE * )
  // @param pEnv   environment to be dumped
  // @param pFilter  if not null, filters output
  procedure uno_dumpEnvironment(stream: Pointer;  pEnv: p_uno_Environment;  pFilter: psal_Char); cdecl;

  // Dumps out environment information, i.e. registered interfaces.
  // @param stream  output stream (FILE * )
  // @param pEnvTypeName type name of environment to be dumped
  // @param pFilter  if not null, filters output
  procedure uno_dumpEnvironmentByName(stream: Pointer;  pEnvTypeName: prtl_uString;  pFilter: psal_Char); cdecl;

implementation

  procedure uno_getEnvironment( var pEnv: p_uno_Environment; pEnvTypeName: prtl_uString; pContext: Pointer); external cppu3LibraryName;
  procedure uno_getRegisteredEnvironments( var ppEnvs: pp_uno_Environment; var nLen: sal_Int32;
                        memAlloc: uno_memAlloc; pEnvTypeName: prtl_uString ); external cppu3LibraryName;
  procedure uno_createEnvironment( var pEnv: p_uno_Environment; pEnvTypeName: prtl_uString; pContext: Pointer ); external cppu3LibraryName;
  procedure uno_dumpEnvironment(stream: Pointer;  pEnv: p_uno_Environment;  pFilter: psal_Char); external cppu3LibraryName;
  procedure uno_dumpEnvironmentByName(stream: Pointer;  pEnvTypeName: prtl_uString;  pFilter: psal_Char); external cppu3LibraryName;

end.
