unit uno_uno_mapping;

interface

{$I uno_common.inc}

uses
   uno_sal_config
  ,uno_rtl_ustring
  ,uno_typelib_typedescription
  ,uno_uno_environment;

type
  p_uno_Mapping = ^_uno_Mapping;
  pp_uno_Mapping = ^p_uno_Mapping;

  // Function pointer declaration to map an interface from one environment to another.
  // @param pMapping   mapping
  // @param ppOut   [inout] destination interface; existing interfaces are released
  // @param pInterface  source interface
  // @param pInterfaceTypeDescr type description of the interface
  uno_MapInterfaceFunc = procedure( pMapping: p_uno_Mapping; var pOut: Pointer; pInterface: Pointer;
                                    pInterfaceTypeDescr: p_typelib_InterfaceTypeDescription ); cdecl;

  // This is the binary specification of a mapping.
  _uno_Mapping = record
    acquire: procedure( pMapping: p_uno_Mapping ); cdecl;
    // Releases mapping. The last release may unload bridges.
    //  @param pMapping    mapping
    release: procedure( pMapping: p_uno_Mapping ); cdecl;
    // mapping function
    mapInterface: uno_MapInterfaceFunc;
  end;
  uno_Mapping = _uno_Mapping;

  // Gets an interface mapping from one environment to another.
  // @param ppMapping [inout] mapping; existing mapping will be released
  // @param pFrom  source environment
  // @param pTo   destination environment
  //    (interfaces resulting in mapInterface() call can be used
  //    in this language environment)
  // @param pAddPurpose additional purpose of mapping (e.g., protocolling); defaults to 0 (none)
  procedure uno_getMapping( var pMapping: p_uno_Mapping; pFrom: p_uno_Environment;
                             pTo: p_uno_Environment; pAddPurpose: prtl_uString ); cdecl;

  // Callback function pointer declaration to get a mapping.
  // @param ppMapping inout mapping
  // @param pFrom  source environment
  // @param pTo   destination environment
  // @param pAddPurpose additional purpose
type
  uno_getMappingFunc = procedure( var pMapping: p_uno_Mapping; pFrom: p_uno_Environment;
                                   pTo: p_uno_Environment; pAddPurpose: prtl_uString ); cdecl;

  // Registers a callback being called each time a mapping is demanded.
  // @param pCallback callback function
  procedure uno_registerMappingCallback( pCallback: uno_getMappingFunc ); cdecl;

  // Revokes a mapping callback registration.
  // @param pCallback callback function
  procedure uno_revokeMappingCallback( pCallback: uno_getMappingFunc ); cdecl;

  // Function pointer declaration to free a mapping.
  // @param pMapping  mapping to be freed
type
    uno_freeMappingFunc = procedure( pMapping: p_uno_Mapping ); cdecl;

  // Registers a mapping. A mapping registers itself on first acquire and revokes itself on last
  //  release. The given freeMapping function is called by the runtime to cleanup any resources.
  // @param ppMapping inout mapping to be registered
  // @param freeMapping called by runtime to delete mapping
  // @param pFrom  source environment
  // @param pTo   destination environment
  // @param pAddPurpose additional purpose string; defaults to 0
  procedure uno_registerMapping( var pMapping: p_uno_Mapping; freeMapping: uno_freeMappingFunc;
                  pFrom: p_uno_Environment; pTo: p_uno_Environment; pAddPurpose: prtl_uString ); cdecl;

  // Revokes a mapping. A mapping registers itself on first acquire and revokes itself on last
  //  release.
  // @param pMapping  mapping to be revoked
  procedure uno_revokeMapping( pMapping: p_uno_Mapping ); cdecl;

  // Gets an interface mapping from one language environment to another by corresponding environment
  //  type names.
  // @param ppMapping [inout] mapping; existing mapping will be released
  // @param pFrom  source environment type name
  // @param pTo   destination environment type name
  //    (interfaces resulting in mapInterface() call can be used
  //    in this language environment)
  // @param pAddPurpose additional purpose of mapping (e.g., protocolling); defaults to 0 (none) *)
  procedure uno_getMappingByName( var pMapping: p_uno_Mapping; pFrom: p_uno_Environment;
                                   pTo: p_uno_Environment; pAddPurpose: prtl_uString ); cdecl;

  // symbol exported by each language binding library
const
  UNO_EXT_GETMAPPING = 'uno_ext_getMapping';

  // Function pointer declaration to get a mapping from a loaded bridge. Bridges export a function
  //  called uno_ext_getMapping() of this signature.
  // @param ppMapping [inout] mapping; existing mapping will be released
  // @pFrom    source environment
  // @pTo    destination environment
type
  uno_ext_getMappingFunc = procedure( var pMapping: p_uno_Mapping; pFrom: p_uno_Environment; pTo: p_uno_Environment ); cdecl;

implementation

  procedure uno_getMapping( var pMapping: p_uno_Mapping; pFrom: p_uno_Environment; pTo: p_uno_Environment; pAddPurpose: prtl_uString ); external cppu3LibraryName;
  procedure uno_registerMappingCallback( pCallback: uno_getMappingFunc ); external cppu3LibraryName;
  procedure uno_revokeMappingCallback( pCallback: uno_getMappingFunc ); external cppu3LibraryName;
  procedure uno_registerMapping( var pMapping: p_uno_Mapping; freeMapping: uno_freeMappingFunc; pFrom: p_uno_Environment; pTo: p_uno_Environment; pAddPurpose: prtl_uString ); external cppu3LibraryName;
  procedure uno_revokeMapping( pMapping: p_uno_Mapping ); external cppu3LibraryName;
  procedure uno_getMappingByName( var pMapping: p_uno_Mapping; pFrom: p_uno_Environment; pTo: p_uno_Environment; pAddPurpose: prtl_uString ); external cppu3LibraryName;


end.
